@@ -7,11 +7,16 @@
 ** place of a destructor.
 *****************************************************************************/
 
+#ifndef KCM_SAMBACONF_JOINDOMAINDLG_UI_H
+#define KCM_SAMBACONF_JOINDOMAINDLG_UI_H
+
 void JoinDomainDlg::accept() {
     if (passwordEdit->text() != verifyEdit->text()) {
-        KMessageBox::sorry(this, "Sorry", "You entered two different passwords. Please try again.");
+        KMessageBox::sorry(this, i18n("Sorry"), i18n("You entered two different passwords. Please try again."));
        return;
     }
     
     QDialog::accept();
 }
+
+#endif // KCM_SAMBACONF_JOINDOMAINDLG_UI_H
