@@ -24,7 +24,15 @@
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kglobal.h>
+#include <kconfig.h>
+#include <kconfiggroup.h>
 #include <qdir.h>
+#include <solid/device.h>
+#include <solid/devicenotifier.h>
+#include <solid/deviceinterface.h>
+#include <solid/video.h>
+
 
 #include "videodevice.h"
 #include "videodevicepool.h"
@@ -40,19 +48,22 @@ __u64 VideoDevicePool::m_clients = 0;
 
 VideoDevicePool* VideoDevicePool::self()
 {
-	kdDebug(14010) << "libkopete (avdevice): self() called" << endl;
+//	kDebug() << "libkopete (avdevice): self() called";
 	if (s_self == NULL)
 	{
 		s_self = new VideoDevicePool;
 		if (s_self)
 			m_clients = 0;
 	}
-	kdDebug(14010) << "libkopete (avdevice): self() exited successfuly. m_clients = " << m_clients << endl;
+//	kDebug() << "libkopete (avdevice): self() exited successfuly";
 	return s_self;
 }
 
 VideoDevicePool::VideoDevicePool()
+: m_current_device(0)
 {
+	connect( Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString&)), SLOT(deviceAdded(const QString &)) );
+	connect( Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString&)), SLOT(deviceRemoved(const QString &)) );
 }
 
 
@@ -64,61 +75,62 @@ VideoDevicePool::~VideoDevicePool()
 
 
 /*!
-    \fn VideoDevicePool::open()
+    \fn VideoDevicePool::open(int device)
  */
-int VideoDevicePool::open()
+int VideoDevicePool::open(int device)
 {
     /// @todo implement me
-
+	kDebug() << "called with device" << device;
 	m_ready.lock();
-	if(!m_videodevice.size())
+	if (!m_videodevice.size())
 	{
-		kdDebug(14010) <<  k_funcinfo << "open(): No devices found. Must scan for available devices." << m_current_device << endl;
+		kDebug() << "open(): No devices found. Must scan for available devices." << m_current_device;
 		scanDevices();
 	}
-	if(!m_videodevice.size())
+	if (!m_videodevice.size() || (device >= m_videodevice.size()))
 	{
-		kdDebug(14010) <<  k_funcinfo << "open(): No devices found. bailing out." << m_current_device << endl;
+		kDebug() << "open(): Device not found. bailing out." << m_current_device;
 		m_ready.unlock();
 		return EXIT_FAILURE;
 	}
-	if(m_current_device >= m_videodevice.size())
+	int current_device = m_current_device;
+	if (device < 0)
 	{
-		kdDebug(14010) <<  k_funcinfo << "open(): Device out of scope (" << m_current_device << "). Defaulting to the first one." << endl;
-		m_current_device = 0;
+		kDebug() << "Trying to load saved device (using default device if not available)";
+		loadSelectedDevice();	// Set m_current_device to saved device (if device available)
 	}
-	int isopen = m_videodevice[currentDevice()].open();
-	if ( isopen == EXIT_SUCCESS)
+	else
+		m_current_device = device;
+	int isopen = EXIT_FAILURE;
+	if ((m_current_device != current_device) || !isOpen())
 	{
-		loadConfig(); // Temporary hack. The open() seems to clean the input parameters. Need to find a way to fix it.
-		
+		if (isOpen())
+		{
+			if (EXIT_SUCCESS == m_videodevice[current_device].close())
+				m_clients--;
+			else
+			{
+				m_ready.unlock();
+				return EXIT_FAILURE;
+			}
+		}
+		isopen = m_videodevice[m_current_device].open();
+		if (isopen == EXIT_SUCCESS)
+		{
+			loadDeviceConfig(); // Load and apply device parameters
+			m_clients++;
+		}
 	}
-	m_clients++;
-	kdDebug(14010) << k_funcinfo << "Number of clients: " << m_clients << endl;
+	else
+	{
+		isopen = EXIT_SUCCESS;
+		m_clients++;
+	}
+	kDebug() << "Number of clients: " << m_clients;
 	m_ready.unlock();
 	return isopen;
 }
 
-/*!
-    \fn VideoDevicePool::open(int device)
- */
-int VideoDevicePool::open(unsigned int device)
-{
-    /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "open(" << device << ") called." << endl;
-	if(device >= m_videodevice.size())
-	{
-		kdDebug(14010) <<  k_funcinfo << "open(" << device <<"): Device does not exist." << endl;
-		return EXIT_FAILURE;
-	}
-	close();
-	kdDebug(14010) <<  k_funcinfo << "open(" << device << ") Setting m_current_Device to " << device << endl;
-	m_current_device = device;
-	saveConfig();
-	kdDebug(14010) <<  k_funcinfo << "open(" << device << ") Calling open()." << endl;
-	return open();
-}
-
 bool VideoDevicePool::isOpen()
 {
 	return m_videodevice[currentDevice()].isOpen();
@@ -168,12 +180,12 @@ int VideoDevicePool::setSize( int newwidth, int newheight)
 		return m_videodevice[currentDevice()].setSize(newwidth, newheight);
 	else
 	{
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::setSize() fallback for no device." << endl;
+		kDebug() << "VideoDevicePool::setSize() fallback for no device.";
 		m_buffer.width=newwidth;
 		m_buffer.height=newheight;
 		m_buffer.pixelformat=	PIXELFORMAT_RGB24;
 		m_buffer.data.resize(m_buffer.width*m_buffer.height*3);
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::setSize() buffer size: "<< m_buffer.data.size() << endl;
+		kDebug() << "VideoDevicePool::setSize() buffer size: "<< m_buffer.data.size();
 	}
 	return EXIT_SUCCESS;
 }
@@ -189,9 +201,9 @@ int VideoDevicePool::close()
 	if((currentDevice() < m_videodevice.size())&&(!m_clients))
 		return m_videodevice[currentDevice()].close();
 	if(m_clients)
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::close() The video device is still in use." << endl;
+		kDebug() << "VideoDevicePool::close() The video device is still in use.";
 	if(currentDevice() >= m_videodevice.size())
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::close() Current device out of range." << endl;
+		kDebug() << "VideoDevicePool::close() Current device out of range.";
 	return EXIT_FAILURE;
 }
 
@@ -200,7 +212,7 @@ int VideoDevicePool::close()
  */
 int VideoDevicePool::startCapturing()
 {
-	kdDebug(14010) <<  k_funcinfo << "startCapturing() called." << endl;
+	kDebug() << "startCapturing() called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].startCapturing();
 	return EXIT_FAILURE;
@@ -343,7 +355,7 @@ bool VideoDevicePool::getAutoBrightnessContrast()
  */
 bool VideoDevicePool::setAutoBrightnessContrast(bool brightnesscontrast)
 {
-	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::setAutoBrightnessContrast(" << brightnesscontrast << ") called." << endl;
+	kDebug() << "VideoDevicePool::setAutoBrightnessContrast(" << brightnesscontrast << ") called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].setAutoBrightnessContrast(brightnesscontrast);
 	return false;
@@ -364,7 +376,7 @@ bool VideoDevicePool::getAutoColorCorrection()
  */
 bool VideoDevicePool::setAutoColorCorrection(bool colorcorrection)
 {
-	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::setAutoColorCorrection(" << colorcorrection << ") called." << endl;
+	kDebug() << "VideoDevicePool::setAutoColorCorrection(" << colorcorrection << ") called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].setAutoColorCorrection(colorcorrection);
 	return false;
@@ -385,7 +397,7 @@ bool VideoDevicePool::getImageAsMirror()
  */
 bool VideoDevicePool::setImageAsMirror(bool imageasmirror)
 {
-	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::setImageAsMirror(" << imageasmirror << ") called." << endl;
+	kDebug() << "VideoDevicePool::setImageAsMirror(" << imageasmirror << ") called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].setImageAsMirror(imageasmirror);
 	return false;
@@ -396,20 +408,21 @@ bool VideoDevicePool::setImageAsMirror(bool imageasmirror)
  */
 int VideoDevicePool::getFrame()
 {
-//	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getFrame() called." << endl;
+//	kDebug() << "VideoDevicePool::getFrame() called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].getFrame();
 	else
 	{
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getFrame() fallback for no device." << endl;
-		for(unsigned int loop=0; loop < m_buffer.data.size(); loop+=3)
+		kDebug() << "VideoDevicePool::getFrame() fallback for no device.";
+		for(int loop=0; loop < m_buffer.data.size(); loop+=3)
 		{
 			m_buffer.data[loop]   = 255;
 			m_buffer.data[loop+1] = 0;
 			m_buffer.data[loop+2] = 0;
 		}
 	}
-//	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getFrame() exited successfuly." << endl;
+	kDebug() << "VideoDevicePool::getFrame() exited successfuly.";
+
 	return EXIT_SUCCESS;
 }
 
@@ -418,13 +431,17 @@ int VideoDevicePool::getFrame()
  */
 int VideoDevicePool::getImage(QImage *qimage)
 {
-//	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getImage() called." << endl;
+//	kDebug() << "VideoDevicePool::getImage() called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].getImage(qimage);
 	else
 	{
-		kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getImage() fallback for no device." << endl;
-		qimage->create(m_buffer.width, m_buffer.height,32, QImage::IgnoreEndian);
+		kDebug() << "VideoDevicePool::getImage() fallback for no device.";
+
+		// do NOT delete qimage here, as it is received as a parameter
+		if (qimage->width() != width() || qimage->height() != height())
+			*qimage = QImage(width(), height(), QImage::Format_RGB32);
+
 		uchar *bits=qimage->bits();
 		switch(m_buffer.pixelformat)
 		{
@@ -437,7 +454,7 @@ int VideoDevicePool::getImage(QImage *qimage)
 			case PIXELFORMAT_RGB565X: break;
 			case PIXELFORMAT_RGB24	:
 				{
-					kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getImage() fallback for no device - RGB24." << endl;
+					kDebug() << "VideoDevicePool::getImage() fallback for no device - RGB24.";
 					int step=0;
 					for(int loop=0;loop < qimage->numBytes();loop+=4)
 					{
@@ -465,9 +482,14 @@ int VideoDevicePool::getImage(QImage *qimage)
 			case PIXELFORMAT_RGB32	: memcpy(bits,&m_buffer.data[0], m_buffer.data.size());
 				break;
 			case PIXELFORMAT_BGR32	: break;
+			case PIXELFORMAT_YUYV   : break;
+			case PIXELFORMAT_UYVY   : break;
+			case PIXELFORMAT_YUV420P: break;
+			case PIXELFORMAT_YUV422P: break;
+			default: break;
 		}
 	}
-	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::getImage() exited successfuly." << endl;
+	kDebug() << "VideoDevicePool::getImage() exited successfuly.";
 	return EXIT_SUCCESS;
 }
 
@@ -476,7 +498,7 @@ int VideoDevicePool::getImage(QImage *qimage)
  */
 int VideoDevicePool::selectInput(int newinput)
 {
-	kdDebug(14010) <<  k_funcinfo << "VideoDevicePool::selectInput(" << newinput << ") called." << endl;
+	kDebug() << "VideoDevicePool::selectInput(" << newinput << ") called.";
 	if(m_videodevice.size())
 		return m_videodevice[currentDevice()].selectInput(newinput);
 	else
@@ -500,17 +522,24 @@ int VideoDevicePool::setInputParameters()
 int VideoDevicePool::fillDeviceKComboBox(KComboBox *combobox)
 {
     /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "fillInputKComboBox: Called." << endl;
-	combobox->clear();
-	if(m_videodevice.size())
+	kDebug() << "Called.";
+// check if KComboBox is a valid pointer.
+	if (combobox != NULL)
 	{
-		for (unsigned int loop=0; loop < m_videodevice.size(); loop++)
+		combobox->clear();
+		kDebug() << "Combobox cleaned.";
+		if(m_videodevice.size())
 		{
-			combobox->insertItem(m_videodevice[loop].m_name);
-			kdDebug(14010) <<  k_funcinfo << "DeviceKCombobox: Added device " << loop << ": " << m_videodevice[loop].m_name << endl;
+			for (int loop=0; loop < m_videodevice.size(); loop++)
+			{
+				combobox->addItem(m_videodevice[loop].m_name);
+				kDebug() << "Added device " << loop << ": " << m_videodevice[loop].m_name;
+			}
+			combobox->setCurrentIndex(currentDevice());
+			combobox->setEnabled(true);
+			return EXIT_SUCCESS;
 		}
-		combobox->setCurrentItem(currentDevice());
-		return EXIT_SUCCESS;
+		combobox->setEnabled(false);
 	}
 	return EXIT_FAILURE;
 }
@@ -521,20 +550,25 @@ int VideoDevicePool::fillDeviceKComboBox(KComboBox *combobox)
 int VideoDevicePool::fillInputKComboBox(KComboBox *combobox)
 {
     /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "fillInputKComboBox: Called." << endl;
-	combobox->clear();
-	if(m_videodevice.size())
+	kDebug() << "Called.";
+	if (combobox != NULL)
 	{
-		if(m_videodevice[currentDevice()].inputs()>0)
+		combobox->clear();
+		if ( !m_videodevice.isEmpty() && (currentDevice()>=0) && currentDevice() < m_videodevice.size() )
 		{
-			for (unsigned int loop=0; loop < m_videodevice[currentDevice()].inputs(); loop++)
+			if(m_videodevice[currentDevice()].inputs()>0)
 			{
-				combobox->insertItem(m_videodevice[currentDevice()].m_input[loop].name);
-				kdDebug(14010) <<  k_funcinfo << "InputKCombobox: Added input " << loop << ": " << m_videodevice[currentDevice()].m_input[loop].name << " (tuner: " << m_videodevice[currentDevice()].m_input[loop].hastuner << ")" << endl;
+				for (int loop=0; loop < m_videodevice[currentDevice()].inputs(); loop++)
+				{
+					combobox->addItem(m_videodevice[currentDevice()].m_input[loop].name);
+					kDebug() << "Added input " << loop << ": " << m_videodevice[currentDevice()].m_input[loop].name << " (tuner: " << m_videodevice[currentDevice()].m_input[loop].hastuner << ")";
+				}
+				combobox->setCurrentIndex(currentInput());
+				combobox->setEnabled(true);
+				return EXIT_SUCCESS;
 			}
-			combobox->setCurrentItem(currentInput());
-			return EXIT_SUCCESS;
 		}
+		combobox->setEnabled(false);
 	}
 	return EXIT_FAILURE;
 }
@@ -545,16 +579,18 @@ int VideoDevicePool::fillInputKComboBox(KComboBox *combobox)
 int VideoDevicePool::fillStandardKComboBox(KComboBox *combobox)
 {
     /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "fillInputKComboBox: Called." << endl;
-	combobox->clear();
-	if(m_videodevice.size())
+	kDebug() << "Called.";
+	if (combobox != NULL)
 	{
-		if(m_videodevice[currentDevice()].inputs()>0)
+		combobox->clear();
+		if ( !m_videodevice.isEmpty() && currentDevice() < m_videodevice.size() )
 		{
-			for (unsigned int loop=0; loop < 25; loop++)
+			if(m_videodevice[currentDevice()].inputs()>0)
 			{
-				if ( (m_videodevice[currentDevice()].m_input[currentInput()].m_standards) & (1 << loop) )
-					combobox->insertItem(m_videodevice[currentDevice()].signalStandardName( 1 << loop));
+				for (unsigned int loop=0; loop < 25; loop++)
+				{
+					if ( (m_videodevice[currentDevice()].m_input[currentInput()].m_standards) & (1 << loop) )
+						combobox->addItem(m_videodevice[currentDevice()].signalStandardName( 1 << loop));
 /*
 				case STANDARD_PAL_B1	: return V4L2_STD_PAL_B1;	break;
 				case STANDARD_PAL_G	: return V4L2_STD_PAL_G;	break;
@@ -591,11 +627,14 @@ int VideoDevicePool::fillStandardKComboBox(KComboBox *combobox)
 				case STANDARD_ALL	: return V4L2_STD_ALL;		break;
 
 				combobox->insertItem(m_videodevice[currentDevice()].m_input[loop].name);
-				kdDebug(14010) <<  k_funcinfo << "StandardKCombobox: Added input " << loop << ": " << m_videodevice[currentDevice()].m_input[loop].name << " (tuner: " << m_videodevice[currentDevice()].m_input[loop].hastuner << ")" << endl;*/
+				kDebug() << "StandardKCombobox: Added input " << loop << ": " << m_videodevice[currentDevice()].m_input[loop].name << " (tuner: " << m_videodevice[currentDevice()].m_input[loop].hastuner << ")";*/
+				}
+				combobox->setCurrentIndex(0);	// FIXME: set to actual signal standard
+				combobox->setEnabled(combobox->count());
+				return EXIT_SUCCESS;
 			}
-			combobox->setCurrentItem(currentInput());
-			return EXIT_SUCCESS;
 		}
+		combobox->setEnabled(false);
 	}
 	return EXIT_FAILURE;
 }
@@ -607,101 +646,60 @@ int VideoDevicePool::scanDevices()
 {
     /// @todo implement me
 
-	kdDebug(14010) <<  k_funcinfo << "called" << endl;
+	if (m_videodevice.isEmpty()) {
+		kDebug() << "called";
 #if defined(__linux__) && defined(ENABLE_AV)
-	QDir videodevice_dir;
-	const QString videodevice_dir_path=QString::fromLocal8Bit("/dev/v4l/");
-	const QString videodevice_dir_filter=QString::fromLocal8Bit("video*");
-	VideoDevice videodevice;
-
-	m_videodevice.clear();
-	m_modelvector.clear();
-
-	videodevice_dir.setPath(videodevice_dir_path);
-	videodevice_dir.setNameFilter(videodevice_dir_filter);
-        videodevice_dir.setFilter( QDir::System | QDir::NoSymLinks | QDir::Readable | QDir::Writable );
-        videodevice_dir.setSorting( QDir::Name );
-
-	kdDebug(14010) <<  k_funcinfo << "Looking for devices in " << videodevice_dir_path << endl;
-	const QFileInfoList *list = videodevice_dir.entryInfoList();
-
-	if (!list)
-	{
-		kdDebug(14010) << k_funcinfo << "Found no suitable devices in " << videodevice_dir_path << endl;
-		QDir videodevice_dir;
-		const QString videodevice_dir_path=QString::fromLocal8Bit("/dev/");
-		const QString videodevice_dir_filter=QString::fromLocal8Bit("video*");
-		VideoDevice videodevice;
-
-		videodevice_dir.setPath(videodevice_dir_path);
-		videodevice_dir.setNameFilter(videodevice_dir_filter);
-        	videodevice_dir.setFilter( QDir::System | QDir::NoSymLinks | QDir::Readable | QDir::Writable );
-        	videodevice_dir.setSorting( QDir::Name );
-
-		kdDebug(14010) <<  k_funcinfo << "Looking for devices in " << videodevice_dir_path << endl;
-		const QFileInfoList *list = videodevice_dir.entryInfoList();
-
-		if (!list)
-		{
-			kdDebug(14010) << k_funcinfo << "Found no suitable devices in " << videodevice_dir_path << endl;
-			return EXIT_FAILURE;
+		foreach (Solid::Device device,
+				Solid::Device::listFromType(Solid::DeviceInterface::Video, QString())) {
+			registerDevice( device );
 		}
 
-		QFileInfoListIterator fileiterator ( *list );
-		QFileInfo *fileinfo;
-
-		kdDebug(14010) <<  k_funcinfo << "scanning devices in " << videodevice_dir_path << "..." << endl;
-		while ( (fileinfo = fileiterator.current()) != 0 )
-		{
-			videodevice.setFileName(fileinfo->absFilePath());
-			kdDebug(14010) <<  k_funcinfo << "Found device " << videodevice.full_filename << endl;
-			videodevice.open(); // It should be opened with O_NONBLOCK (it's a FIFO) but I dunno how to do it using QFile
-			if(videodevice.isOpen())
-			{
-				kdDebug(14010) <<  k_funcinfo << "File " << videodevice.full_filename << " was opened successfuly" << endl;
-
-// This must be changed to proper code to handle multiple devices of the same model. It currently simply add models without proper checking
-				videodevice.close();
-				videodevice.m_modelindex=m_modelvector.addModel (videodevice.m_model); // Adds device to the device list and sets model number
-				m_videodevice.push_back(videodevice);
-			}
-			++fileiterator;
-		}
-
-
-		m_current_device = 0;
-		loadConfig();
-		kdDebug(14010) <<  k_funcinfo << "exited successfuly" << endl;
-		return EXIT_SUCCESS;
-
-	}
-	QFileInfoListIterator fileiterator ( *list );
-	QFileInfo *fileinfo;
-
-	kdDebug(14010) <<  k_funcinfo << "scanning devices in " << videodevice_dir_path << "..." << endl;
-	while ( (fileinfo = fileiterator.current()) != 0 )
-	{
-		videodevice.setFileName(fileinfo->absFilePath());
-		kdDebug(14010) <<  k_funcinfo << "Found device " << videodevice.full_filename << endl;
-		videodevice.open(); // It should be opened with O_NONBLOCK (it's a FIFO) but I dunno how to do it using QFile
-		if(videodevice.isOpen())
-		{
-			kdDebug(14010) <<  k_funcinfo << "File " << videodevice.full_filename << " was opened successfuly" << endl;
-
-// This must be changed to proper code to handle multiple devices of the same model. It currently simply add models without proper checking
-			videodevice.close();
-			videodevice.m_modelindex=m_modelvector.addModel (videodevice.m_model); // Adds device to the device list and sets model number
-			m_videodevice.push_back(videodevice);
-		}
-		++fileiterator;
-	}
-	m_current_device = 0;
-	loadConfig();
 #endif
-	kdDebug(14010) <<  k_funcinfo << "exited successfuly" << endl;
+		kDebug() << "exited successfuly";
+	} else {
+		kDebug() << "Not scanning: initial device list already loaded";
+	}
 	return EXIT_SUCCESS;
 }
 
+void VideoDevicePool::registerDevice( Solid::Device & device )
+{
+	kDebug() << "New video device at " << device.udi();
+	const Solid::Device * vendorDevice = &device;
+	while ( vendorDevice->isValid() && vendorDevice->vendor().isEmpty() )
+	{
+		vendorDevice = new Solid::Device( vendorDevice->parentUdi() );
+	}
+	if ( vendorDevice->isValid() )
+	{
+		kDebug() << "vendor: " << vendorDevice->vendor() << ", product: " << vendorDevice->product();
+	}
+	Solid::Video * solidVideoDevice = device.as<Solid::Video>();
+	if ( solidVideoDevice ) {
+		QStringList protocols = solidVideoDevice->supportedProtocols();
+		if ( protocols.contains( "video4linux" ) )
+		{
+			QStringList drivers = solidVideoDevice->supportedDrivers( "video4linux" );
+			if ( drivers.contains( "video4linux" ) )
+			{
+				kDebug() << "V4L device path is" << solidVideoDevice->driverHandle( "video4linux" ).toString();
+				VideoDevice videodevice;
+				videodevice.setUdi( device.udi() );
+				videodevice.setFileName(solidVideoDevice->driverHandle( "video4linux" ).toString());
+				kDebug() << "Found device " << videodevice.full_filename;
+				videodevice.open();
+				if(videodevice.isOpen())
+				{
+					kDebug() << "File " << videodevice.full_filename << " was opened successfuly";
+					videodevice.close();
+					videodevice.m_modelindex=m_modelvector.addModel (videodevice.m_model); // Adds device to the device list and sets model number
+					m_videodevice.push_back(videodevice);
+				}
+			}
+		}
+	}
+}
+
 /*!
     \fn Kopete::AV::VideoDevicePool::hasDevices()
  */
@@ -722,10 +720,21 @@ size_t VideoDevicePool::size()
 	return m_videodevice.size();
 }
 
+/*!
+    \fn Kopete::AV::VideoDevicePool::currentDeviceUdi()
+ */
+QString VideoDevicePool::currentDeviceUdi()
+{
+	if (m_videodevice.size() && (m_current_device >= 0))
+		return m_videodevice[m_current_device].udi();
+	else
+		return QString();
+}
+
 /*!
     \fn Kopete::AV::VideoDevicePool::currentDevice()
  */
-unsigned int VideoDevicePool::currentDevice()
+int VideoDevicePool::currentDevice()
 {
     /// @todo implement me
 	return m_current_device;
@@ -750,18 +759,16 @@ unsigned int VideoDevicePool::inputs()
 }
 
 /*!
-    \fn Kopete::AV::VideoDevicePool::loadConfig()
+    \fn Kopete::AV::VideoDevicePool::loadSelectedDevice()
  */
-void VideoDevicePool::loadConfig()
+void VideoDevicePool::loadSelectedDevice()
 {
-    /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "called" << endl;
-	if((hasDevices())&&(m_clients==0))
+	kDebug() << "called";
+	if (hasDevices())
 	{
-		KConfig *config = KGlobal::config();
-		config->setGroup("Video Device Settings");
-		const QString currentdevice = config->readEntry("Current Device", QString::null);
-		kdDebug(14010) << k_funcinfo << "Current device: " << currentdevice << endl;
+		KConfigGroup config(KGlobal::config(), "Video Device Settings");
+		QString currentdevice = config.readEntry("Current Device", QString());
+		kDebug() << "Current device: " << currentdevice;
 
 //		m_current_device = 0; // Must check this thing because of the fact that multiple loadConfig in other methodas can do bad things. Watch out!
 
@@ -771,43 +778,66 @@ void VideoDevicePool::loadConfig()
 			const QString modelindex = QString::fromLocal8Bit ( "Model %1 Device %2")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex);
 			if(modelindex == currentdevice)
 			{
-				m_current_device = vditerator - m_videodevice.begin();
-//				kdDebug(14010) << k_funcinfo << "This place will be used to set " << modelindex << " as the current device ( " << (vditerator - m_videodevice.begin()) << " )." << endl;
+				m_current_device = std::distance (m_videodevice.begin(), vditerator);
+//				kDebug() << "This place will be used to set " << modelindex << " as the current device ( " << std::distance(m_videodevice.begin(), vditerator ) << " ).";
 			}
-			const QString name                = config->readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Name")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex)), (*vditerator).m_model);
-			const int currentinput            = config->readNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Current input")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex)), 0);
-			kdDebug(14010) << k_funcinfo << "Device name: " << name << endl;
-			kdDebug(14010) << k_funcinfo << "Device current input: " << currentinput << endl;
+			const QString name                = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Name")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex)), (*vditerator).m_model);
+			kDebug() << "Device name: " << name;
+		}
+	}
+}
+
+/*!
+    \fn Kopete::AV::VideoDevicePool::loadDeviceConfig()
+ */
+void VideoDevicePool::loadDeviceConfig()
+{
+    /// @todo implement me
+	kDebug() << "called";
+	if (hasDevices())
+	{
+		KConfigGroup config(KGlobal::config(), "Video Device Settings");
+
+		VideoDeviceVector::iterator vditerator;
+		for( vditerator = m_videodevice.begin(); vditerator != m_videodevice.end(); ++vditerator )
+		{
+			const QString modelindex = QString::fromLocal8Bit ( "Model %1 Device %2")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex);
+			const QString name                = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Name")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex)), (*vditerator).m_model);
+			const int currentinput            = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Current input")  .arg ((*vditerator).m_name ) .arg ((*vditerator).m_modelindex)), 0);
+			kDebug() << "Device name: " << name;
+			kDebug() << "Device current input: " << currentinput;
 			(*vditerator).selectInput(currentinput);
 
-			for (size_t input = 0 ; input < (*vditerator).m_input.size(); input++)
+			for (int input = 0; input < (*vditerator).m_input.size(); input++)
 			{
-				const float brightness = config->readDoubleNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Brightness").arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
-				const float contrast   = config->readDoubleNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Contrast")  .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
-				const float saturation = config->readDoubleNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Saturation").arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
-				const float whiteness  = config->readDoubleNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Whiteness") .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
-				const float hue        = config->readDoubleNumEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Hue")       .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
-				const bool  autobrightnesscontrast = config->readBoolEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoBrightnessContrast") .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
-				const bool  autocolorcorrection    = config->readBoolEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoColorCorrection")    .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
-				const bool  imageasmirror          = config->readBoolEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 mageAsMirror")           .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
+				const float brightness = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Brightness").arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
+				const float contrast   = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Contrast")  .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
+				const float saturation = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Saturation").arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
+				const float whiteness  = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Whiteness") .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
+				const float hue        = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Hue")       .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , 0.5 );
+				const bool  autobrightnesscontrast = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoBrightnessContrast") .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
+				const bool  autocolorcorrection    = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoColorCorrection")    .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
+				const bool  imageasmirror          = config.readEntry((QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 ImageAsMirror")          .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input)) , false );
 				(*vditerator).setBrightness(brightness);
 				(*vditerator).setContrast(contrast);
 				(*vditerator).setSaturation(saturation);
+				(*vditerator).setWhiteness(whiteness);
 				(*vditerator).setHue(hue);
 				(*vditerator).setAutoBrightnessContrast(autobrightnesscontrast);
 				(*vditerator).setAutoColorCorrection(autocolorcorrection);
 				(*vditerator).setImageAsMirror(imageasmirror);
-				kdDebug(14010) <<  k_funcinfo << "Brightness:" << brightness << endl;
-				kdDebug(14010) <<  k_funcinfo << "Contrast  :" << contrast   << endl;
-				kdDebug(14010) <<  k_funcinfo << "Saturation:" << saturation << endl;
-				kdDebug(14010) <<  k_funcinfo << "Whiteness :" << whiteness << endl;
-				kdDebug(14010) <<  k_funcinfo << "Hue       :" << hue        << endl;
-				kdDebug(14010) <<  k_funcinfo << "AutoBrightnessContrast:" << autobrightnesscontrast << endl;
-				kdDebug(14010) <<  k_funcinfo << "AutoColorCorrection   :" << autocolorcorrection    << endl;
-				kdDebug(14010) <<  k_funcinfo << "ImageAsMirror         :" << imageasmirror          << endl;
+				kDebug() << "Brightness:" << brightness;
+				kDebug() << "Contrast  :" << contrast;
+				kDebug() << "Saturation:" << saturation;
+				kDebug() << "Whiteness :" << whiteness;
+				kDebug() << "Hue       :" << hue;
+				kDebug() << "AutoBrightnessContrast:" << autobrightnesscontrast;
+				kDebug() << "AutoColorCorrection   :" << autocolorcorrection;
+				kDebug() << "ImageAsMirror         :" << imageasmirror;
 			}
 		}
 	}
+	/* TODO: load and apply signal standard */
 }
 
 /*!
@@ -816,47 +846,46 @@ void VideoDevicePool::loadConfig()
 void VideoDevicePool::saveConfig()
 {
     /// @todo implement me
-	kdDebug(14010) <<  k_funcinfo << "called" << endl;
+	kDebug() << "called";
 	if(hasDevices())
 	{
-		KConfig *config = KGlobal::config();
-		config->setGroup("Video Device Settings");
+		KConfigGroup config(KGlobal::config(), "Video Device Settings");
 
 /*		if(m_modelvector.size())
 		{
 			VideoDeviceModelPool::m_devicemodel::iterator vmiterator;
 			for( vmiterator = m_modelvector.begin(); vmiterator != m_modelvector.end(); ++vmiterator )
 			{
-				kdDebug(14010) << "Device Model: " << (*vmiterator).model << endl;
-				kdDebug(14010) << "Device Count: " << (*vmiterator).count << endl;
+				kDebug() << "Device Model: " << (*vmiterator).name;
+				kDebug() << "Device Count: " << (*vmiterator).count;
 			}
 		}
 */
 // Stores what is the current video device in use
 		const QString currentdevice = QString::fromLocal8Bit ( "Model %1 Device %2" ) .arg(m_videodevice[m_current_device].m_model) .arg(m_videodevice[m_current_device].m_modelindex);
-		config->writeEntry( "Current Device", currentdevice);
+		config.writeEntry( "Current Device", currentdevice);
 
 		VideoDeviceVector::iterator vditerator;
 		for( vditerator = m_videodevice.begin(); vditerator != m_videodevice.end(); ++vditerator )
 		{
-			kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Name:" << (*vditerator).m_name << endl;
-			kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Current input:" << (*vditerator).currentInput() << endl;
+			kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Name:" << (*vditerator).m_name;
+			kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Current input:" << (*vditerator).currentInput();
 
 // Stores current input for the given video device
 			const QString name                   = QString::fromLocal8Bit ( "Model %1 Device %2 Name")  .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex);
 			const QString currentinput           = QString::fromLocal8Bit ( "Model %1 Device %2 Current input")  .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex);
-			config->writeEntry( name,                   (*vditerator).m_name);
-			config->writeEntry( currentinput,           (*vditerator).currentInput());
+			config.writeEntry( name,                   (*vditerator).m_name);
+			config.writeEntry( currentinput,           (*vditerator).currentInput());
 
-			for (size_t input = 0 ; input < (*vditerator).m_input.size(); input++)
+			for (int input = 0 ; input < (*vditerator).m_input.size(); input++)
 			{
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Brightness: " << (*vditerator).m_input[input].getBrightness() << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Contrast  : " << (*vditerator).m_input[input].getContrast()   << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Saturation: " << (*vditerator).m_input[input].getSaturation() << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Whiteness : " << (*vditerator).m_input[input].getWhiteness()  << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Hue       : " << (*vditerator).m_input[input].getHue()        << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Automatic brightness / contrast: " << (*vditerator).m_input[input].getAutoBrightnessContrast() << endl;
-				kdDebug(14010) << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Automatic color correction     : " << (*vditerator).m_input[input].getAutoColorCorrection() << endl;
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Brightness: " << (*vditerator).m_input[input].getBrightness();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Contrast  : " << (*vditerator).m_input[input].getContrast();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Saturation: " << (*vditerator).m_input[input].getSaturation();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Whiteness : " << (*vditerator).m_input[input].getWhiteness();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Hue       : " << (*vditerator).m_input[input].getHue();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Automatic brightness / contrast: " << (*vditerator).m_input[input].getAutoBrightnessContrast();
+				kDebug() << "Model:" << (*vditerator).m_model << ":Index:" << (*vditerator).m_modelindex << ":Input:" << input << ":Automatic color correction     : " << (*vditerator).m_input[input].getAutoColorCorrection();
 
 // Stores configuration about each channel
 				const QString brightness             = QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 Brightness")             .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input);
@@ -867,23 +896,67 @@ void VideoDevicePool::saveConfig()
 				const QString autobrightnesscontrast = QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoBrightnessContrast") .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input);
 				const QString autocolorcorrection    = QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 AutoColorCorrection")    .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input);
 				const QString imageasmirror          = QString::fromLocal8Bit ( "Model %1 Device %2 Input %3 ImageAsMirror")          .arg ((*vditerator).m_model ) .arg ((*vditerator).m_modelindex) .arg (input);
-				config->writeEntry( brightness,             (*vditerator).m_input[input].getBrightness());
-				config->writeEntry( contrast,               (*vditerator).m_input[input].getContrast());
-				config->writeEntry( saturation,             (*vditerator).m_input[input].getSaturation());
-				config->writeEntry( whiteness,              (*vditerator).m_input[input].getWhiteness());
-				config->writeEntry( hue,                    (*vditerator).m_input[input].getHue());
-				config->writeEntry( autobrightnesscontrast, (*vditerator).m_input[input].getAutoBrightnessContrast());
-				config->writeEntry( autocolorcorrection,    (*vditerator).m_input[input].getAutoColorCorrection());
-				config->writeEntry( imageasmirror,          (*vditerator).m_input[input].getImageAsMirror());
+				config.writeEntry( brightness,             (double)(*vditerator).m_input[input].getBrightness());
+				config.writeEntry( contrast,               (double)(*vditerator).m_input[input].getContrast());
+				config.writeEntry( saturation,             (double)(*vditerator).m_input[input].getSaturation());
+				config.writeEntry( whiteness,              (double)(*vditerator).m_input[input].getWhiteness());
+				config.writeEntry( hue,                    (double)(*vditerator).m_input[input].getHue());
+				config.writeEntry( autobrightnesscontrast, (*vditerator).m_input[input].getAutoBrightnessContrast());
+				config.writeEntry( autocolorcorrection,    (*vditerator).m_input[input].getAutoColorCorrection());
+				config.writeEntry( imageasmirror,          (*vditerator).m_input[input].getImageAsMirror());
 			}
 		}
-		config->sync();
-		kdDebug(14010) << endl;
+		config.sync();
+		kDebug();
+	}
+	/* TODO: save signal standard */
+}
+
+void VideoDevicePool::deviceAdded( const QString & udi )
+{
+	kDebug() << "("<< udi << ") called";
+	Solid::Device dev( udi );
+	if ( dev.is<Solid::Video>() )
+	{
+		registerDevice( dev );
+		emit deviceRegistered( udi );
 	}
 }
 
-
-
+void VideoDevicePool::deviceRemoved( const QString & udi )
+{
+	kDebug() << "("<< udi << ") called";
+	int i = 0;
+	m_ready.lock();
+	foreach ( VideoDevice vd, m_videodevice )
+	{
+		if ( vd.udi() == udi )
+		{
+			if (m_current_device == i)
+			{
+				m_current_device = 0;
+				m_clients = 0;
+			}
+			else if (m_current_device > i)
+			{
+				m_current_device--;
+			}
+			m_modelvector.removeModel(m_videodevice[i].m_model);
+			m_videodevice.remove( i );
+			kDebug() << "Video device '" << udi << "' has been removed!";
+			m_ready.unlock();
+			emit deviceUnregistered( udi );
+			/* NOTE: do not emit deviceUnregistered( udi ) with mutex locked ! => potential deadlock ! */
+			return;
+		}
+		else
+		{
+			i++;
+		}
+	}
+	m_ready.unlock();
 }
 
-}
+} // namespace AV
+
+} // namespace Kopete
