@@ -1,10 +1,12 @@
 /*
     kopeteaccount.h - Kopete Account
 
-    Copyright (c) 2003-2004 by Olivier Goffart       <ogoffart@ tiscalinet.be>
+    Copyright (c) 2007      by Gustavo Pichorim Boiko <gustavo.boiko@kdemail.net>
+    Copyright (c) 2003-2005 by Olivier Goffart       <ogoffart@kde.org>
     Copyright (c) 2003-2004 by Martijn Klingens      <klingens@kde.org>
     Copyright (c) 2004      by Richard Smith         <kde@metafoo.co.uk>
-    Kopete    (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>
+
+    Kopete    (c) 2002-2007 by the Kopete developers <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -20,36 +22,38 @@
 #define KOPETEACCOUNT_H
 
 #include "kopeteonlinestatus.h"
+#include "kopetestatusmessage.h"
+#include "kopete_export.h"
 
-#include <kdemacros.h>
+#include <QtCore/QObject>
+#include <QtGui/QPixmap>
+#include <QtCore/QHash>
 
-#include <qobject.h>
-#include <qdict.h>
+#include <kconfiggroup.h>
 
-class QDomNode;
 class KActionMenu;
 class KConfigGroup;
 
 namespace Kopete
 {
 class Contact;
-class Plugin;
 class Protocol;
 class MetaContact;
 class Group;
 class OnlineStatus;
 class BlackLister;
+class StatusMessage;
+class Identity;
+class PropertyContainer;
 
 /**
  * The Kopete::Account class handles one account.
- * Each protocol should subclass this class in its own custom accounts class.
- * There are few pure virtual method that the protocol must implement. Examples are:
+ * Each protocol implementation should subclass this class in its own custom account class.
+ * There are a few pure virtual methods that must be implemented. Examples are:
  * \li \ref connect()
  * \li \ref disconnect()
  * \li \ref createContact()
  *
- * If your account requires a password, derive from @ref PasswordedAccount instead of this class.
- *
  * The accountId is an @em constant unique id, which represents the login.
  * The @ref myself() contact is one of the most important contacts, which represents
  * the user tied to this account. You must create this contact in the contructor of your
@@ -61,7 +65,7 @@ class BlackLister;
  * When you create a new account, you have to register it with the account manager by calling
  * @ref AccountManager::registerAccount.
  *
- * @author Olivier Goffart  <ogoffart @tiscalinet.be>
+ * @author Olivier Goffart  <ogoffart\@kde.org>
  */
 class KOPETE_EXPORT Account : public QObject
 {
@@ -84,9 +88,10 @@ public:
 	 * Manual means that the disconnection was done by the user and no reconnection
 	 * will take place. Any other value will reconnect the account on disconnection.
 	 * The case where the password is wrong will be handled differently.
-	 * @see @ref disconnected
+	 * @see @ref disconnected 
 	 */
 	enum DisconnectReason {
+		OtherClient = -4,    ///< connection went down because another client connected the same account
 		BadPassword = -3,    ///< connection failed because password was incorrect
 		BadUserName = -2,    ///< connection failed because user name was invalid / unknown
 		InvalidHost = -1,    ///< connection failed because host is unreachable
@@ -96,12 +101,38 @@ public:
 	};
 
 	/**
+	 * \brief Describes what should be done when the contact is added to a metacontact
+	 * @sa @ref addContact()
+	 */
+	enum AddMode {
+		ChangeKABC = 0,     ///< The KDE Address book may be updated
+		DontChangeKABC = 1, ///< The KDE Address book will not be changed
+		Temporary = 2       ///< The contact will not be added on the contact list
+	};
+
+	/**
+	 * \brief Describes what should be done when we set new OnlineStatus
+	 * @sa @ref setOnlineStatus()
+	 */
+	enum OnlineStatusOption {
+		None = 0x00,               ///< Use the online status
+		KeepSpecialFlags = 0x01    ///< Use the online status but keep special flags, e.g. Invisible
+	};
+	Q_DECLARE_FLAGS(OnlineStatusOptions, OnlineStatusOption)
+
+	/**
+	 * Constructor for the Account object.
+	 * 
 	 * @param parent the protocol for this account. The account is a child object of the
 	 * protocol, so it will be automatically deleted when the protocol is.
 	 * @param accountID the unique ID of this account.
 	 * @param name the name of this QObject.
 	 */
-	Account(Protocol *parent, const QString &accountID, const char *name=0L);
+	Account(Protocol *parent, const QString &accountID);
+
+	/**
+	 * Destroy the Account object.
+	 */
 	~Account();
 
 	/**
@@ -115,7 +146,10 @@ public:
 	QString accountId() const;
 
 	/**
-	 * \return The label of this account, for the GUI
+	 * The label for this account.
+	 *
+	 * This is used in the GUI.
+	 * \return The label of this account
 	 */
 	QString accountLabel() const;
 
@@ -176,6 +210,44 @@ public:
 	 * @return the icon for this account, colored if needed
 	 */
 	QPixmap accountIcon( const int size = 0 ) const;
+	
+	/**
+	 * \brief change the account icon.
+	 * by default the icon of an account is the protocol one, but it may be overide it.
+	 * Set QString() to go back to the default  (the protocol icon)
+	 * 
+	 * this call will emit colorChanged()
+	 */
+	void setCustomIcon( const QString& );
+	
+	/**
+	 * \brief return the icon base
+	 * This is the custom account icon set with setIcon.   if this icon is null, then the protocol icon is used
+	 * don't use this function to get the icon that need to be displayed, use accountIcon
+	 */
+	QString customIcon() const;
+	
+	/**
+	 * \brief Retrieve the identity this account belongs to
+	 *
+	 * \return a pointer to the Identity object this account belongs to.
+	 *
+	 * \see setIdentity().
+	 */
+	Identity * identity() const;
+
+	/**
+	 * \brief Sets the identity this account belongs to
+	 *
+	 * Setting the account to a new identity implies it to be removed from the 
+	 * identity it was previously associated.
+	 *
+	 * @param ident The identity this account should be associated to
+	 * \return @c true if the identity was changed, @c false otherwise
+	 *
+	 * @note You should call the default implementation from your reimplementation
+	 */
+	virtual bool setIdentity( Kopete::Identity *ident );
 
 	/**
 	 * \brief Retrieve the 'myself' contact.
@@ -187,9 +259,9 @@ public:
 	Contact * myself() const;
 
 	/**
-	 * @brief Return the menu for this account
+	 * @brief Fill the menu with actions for this account
 	 *
-	 * You have to reimplement this method to return the custom action menu which will
+	 * You have to reimplement this method to add custom actions to the @p actionMenu which will
 	 * be shown in the statusbar. It is the caller's responsibility to ensure the menu is deleted.
 	 *
 	 * The default implementation provides a generic menu, with actions generated from the protocol's
@@ -200,16 +272,26 @@ public:
 	 *
 	 * @see OnlineStatusManager::registerOnlineStatus
 	 */
-	virtual KActionMenu* actionMenu() ;
+	virtual void fillActionMenu( KActionMenu *actionMenu );
 
 	/**
-	 * @brief Retrieve the list of contacts for this account
+	 * @brief Return true if account has custom status menu.
+	 *
+	 * You have to reimplement this method and return true if you don't want to have status menu in menu
+	 * which will be shown in the statusbar
+	 *
+	 * The default implementation returns false.
+	 */
+	virtual bool hasCustomStatusMenu() const;
+
+	/**
+	 * @brief Retrieve the list of contacts for this account (except myself contact)
 	 *
 	 * The list is guaranteed to contain only contacts for this account,
 	 * so you can safely use static_cast to your own derived contact class
 	 * if needed.
 	 */
-	const QDict<Contact>& contacts();
+	const QHash<QString,Contact*>& contacts();
 
 	/**
 	 * Indicates whether or not we should suppress status notifications
@@ -222,16 +304,6 @@ public:
 	 */
 	bool suppressStatusNotification() const;
 
-	/**
-	 * \brief Describes what should be done when the contact is added to a metacontact
-	 * @sa @ref addContact()
-	 */
-	enum AddMode {
-		ChangeKABC = 0,     ///< The KDE Address book may be updated
-		DontChangeKABC = 1, ///< The KDE Address book will not be changed
-		Temporary = 2       ///< The contact will not be added on the contactlist
-	};
-
 	/**
 	 * \brief Create a contact (creating a new metacontact if necessary)
 	 *
@@ -244,14 +316,13 @@ public:
 	 * If @p mode is @c DontChangeKABC, no additional action is carried out.
 	 *
 	 * @param contactId the @ref Contact::contactId of the contact to create
-	 * @param displayName the displayname (alias) of the new metacontact. Leave as QString::null if
+	 * @param displayName the displayname (alias) of the new metacontact. Leave as QString() if
 	 *                    no alias is known, then by default, the nick will be taken as alias and tracked if changed.
 	 * @param group the group to add the created metacontact to, or 0 for the top-level group.
 	 * @param mode the mode used to add the contact. Use DontChangeKABC when deserializing.
-	 * @return @c true if creation of the contact succeeded or the contact was already in the list,
-	 *         @c false otherwise.
+	 * @return the new created metacontact or 0L if the operation failed
 	 */
-	bool addContact( const QString &contactId, const QString &displayName = QString::null, Group *group = 0, AddMode mode = DontChangeKABC ) ;
+	MetaContact* addContact( const QString &contactId, const QString &displayName = QString(), Group *group = 0, AddMode mode = DontChangeKABC );
 
 	/**
 	 * @brief Create a new contact, adding it to an existing metacontact
@@ -289,8 +360,10 @@ public:
 	/**
 	 * Return the @ref KConfigGroup used to write and read special properties
 	 *
-	 * "Protocol", "AccountId" , "Color", "AutoConnect", "Priority", "Enabled" are reserved keyword
+	 * "Protocol", "AccountId" , "Color", "AutoConnect", "Priority", "Enabled" , "Icon" are reserved keyword
 	 * already in use in that group.
+	 * 
+	 * for compatibility, try to not use key that start with a uppercase
 	 */
 	KConfigGroup *configGroup() const;
 
@@ -311,11 +384,13 @@ public:
 
 	/**
 	 * \return a pointer to the blacklist of the account
+	 * @todo remove or implement correctly (BlackLister)
 	 */
 	BlackLister* blackLister();
 
 	/**
 	 * \return @c true if the contact with ID @p contactId is in the blacklist, @c false otherwise.
+	 * @todo remove or implement correctly (BlackLister)
 	 */
 	virtual bool isBlocked( const QString &contactId );
 
@@ -330,6 +405,9 @@ protected:
 	 * contact is used as a member of every ChatSession involving this account. myself's
 	 * contactId should be the accountID. The online status of the myself contact represents
 	 * the account's status.
+	 * 
+	 * The myself should have the @ref ContactList::myself() as parent metacontact
+	 * 
 	 */
 	void setMyself( Contact *myself );
 
@@ -365,9 +443,12 @@ protected slots:
 	 * You have to call this method when you are disconnected. Depending on the value of
 	 * @p reason, this function may attempt to reconnect to the server.
 	 *
+	 * - BadPassword will ask again for the password
+	 * - OtherClient will show a message box
+	 *
 	 * @param reason the reason for the disconnection.
 	 */
-	virtual void disconnected( DisconnectReason reason );
+	virtual void disconnected( Kopete::Account::DisconnectReason reason );
 
 	/**
 	 * @brief Sets the online status of all contacts in this account to the same value
@@ -386,6 +467,9 @@ protected slots:
 signals:
 	/**
 	 * The color of the account has been changed
+	 * 
+	 * also emitted when the icon change
+	 * @todo  probably rename to accountIconChanged
 	 */
 	void colorChanged( const QColor & );
 
@@ -413,7 +497,7 @@ public:
 	 * Register a new Contact with the account. This should be called @em only from the
 	 * @ref Contact constructor, not from anywhere else (not even a derived class).
 	 */
-	void registerContact( Contact *c );
+	bool registerContact( Contact *c );
 
 public slots:
 	/**
@@ -421,6 +505,7 @@ public slots:
 	 *
 	 * @param initialStatus is the status to connect with. If it is an invalid status for this
 	 * account, the default online for the account should be used.
+	 * @todo probably deprecate in favor of setOnlineStatus
 	 */
 	virtual void connect( const Kopete::OnlineStatus& initialStatus = OnlineStatus() ) = 0;
 
@@ -429,26 +514,28 @@ public slots:
 	 *
 	 * If the service is connecting, you should abort the connection.
 	 *
-	 * You should call the @ref diconnected function from this function.
+	 * You should call the @ref disconnected function from this function.
+	 * @todo  probably deprecate in favor of setOnlineStatus
 	 */
 	virtual void disconnect( ) = 0 ;
 
 public slots:
-	/**
-	 * If @p away is @c true, set the account away with away message @p reason. Otherwise,
-	 * set the account to not be away.
-	 *
-	 * @todo change ; make use of setOnlineStatus
-	 */
-	virtual void setAway( bool away, const QString &reason = QString::null ) = 0;
-
 	/**
 	 * Reimplement this function to set the online status
 	 * @param status is the new status
-	 * @param reason is the away message to set.
+	 * @param reason is the status message to set.
+	 * @param options specify how the status should be set
 	 * @note If needed, you need to connect.  if the offline status is given, you should disconnect
 	 */
-	virtual void setOnlineStatus( const Kopete::OnlineStatus& status , const QString &reason = QString::null ) = 0;
+	virtual void setOnlineStatus( const Kopete::OnlineStatus& status,
+	                              const Kopete::StatusMessage &reason = Kopete::StatusMessage(),
+	                              const OnlineStatusOptions& options = None ) = 0;
+	/**
+	 * Reimplement this function to set the status message(with metadata).
+	 * You should use this method to set the status message instead of using setOnlineStatus.
+	 * @param statusMessage is the status message to set. (Use Kopete::StatusMessage).
+	 */
+	virtual void setStatusMessage( const Kopete::StatusMessage &statusMessage ) = 0;
 
 	/**
 	 * Display the edit account widget for the account
@@ -460,6 +547,7 @@ public slots:
 	 * blackList()->addContact( contactId )
 	 *
 	 * @param contactId the contact to be added to the blacklist
+	 * @todo remove or implement correctly (BlackLister)
 	 */
 	virtual void block( const QString &contactId );
 
@@ -468,10 +556,16 @@ public slots:
 	 * blackList()->removeContact( contactId )
 	 *
 	 * @param contactId the contact to be removed from the blacklist
+	 * @todo remove or implement correctly (BlackLister)
 	 */
 	virtual void unblock( const QString &contactId );
 
 private slots:
+	/**
+	 * Restore online status and status message on reconnect.
+	 */
+	void reconnect();
+
 	/**
 	 * Track the deletion of a Contact and clean up
 	 */
@@ -482,6 +576,11 @@ private slots:
 	 */
 	void slotOnlineStatusChanged( Kopete::Contact *contact, const Kopete::OnlineStatus &newStatus, const Kopete::OnlineStatus &oldStatus );
 
+	/**
+	 * The @ref myself() contact's property changed.
+	 */
+	void slotContactPropertyChanged( Kopete::PropertyContainer *, const QString &, const QVariant &, const QVariant & );
+
 	/**
 	 * Stop the suppression of status notification (connected to a timer)
 	 */
@@ -489,24 +588,9 @@ private slots:
 
 private:
 	class Private;
-	Private *d;
-
-protected:
-	virtual void virtual_hook( uint id, void* data);
-
-public:
-	/**
-	 * @todo remove
-	 * @deprecated  use configGroup
-	 */
-	void setPluginData( Plugin* /*plugin*/, const QString &key, const QString &value ) KDE_DEPRECATED;
-
-	/**
-	 * @todo remove
-	 * @deprecated  use configGroup
-	 */
-	QString pluginData( Plugin* /*plugin*/, const QString &key ) const KDE_DEPRECATED;
+	Private * const d;
 };
+Q_DECLARE_OPERATORS_FOR_FLAGS(Account::OnlineStatusOptions)
 
 } //END namespace Kopete
 
