@@ -4,7 +4,8 @@
                              -------------------
     begin                : Sat May 25 2005
     copyright            : (C) 2005 by Till Gerken <till@tantalo.net>
-                           (C) 2006 by Michaël Larouche <michael.larouche@kdemail.net>
+                           (C) 2006 by Michaël Larouche <larouche@kde.org>
+    Copyright 2006 by Tommi Rantala <tommi.rantala@cs.helsinki.fi>
 
 			   Kopete (C) 2001-2006 Kopete developers
 			   <kopete-devel@kde.org>.
@@ -21,15 +22,20 @@
 
 #include "jabberclient.h"
 
-#include <qtimer.h>
-#include <qregexp.h>
+#include <kdebug.h>
+#include "kopetesockettimeoutwatcher.h"
+
+#include <QTimer>
+#include <QTcpSocket>
+#include <QRegExp>
+#include <QtCrypto>
 
-#include <qca.h>
 #include <bsocket.h>
 #include <filetransfer.h>
+#include <jinglesessionmanager.h>
 #include <xmpp_tasks.h>
 
-#include "jabberconnector.h"
+#include "privacymanager.h"
 
 #define JABBER_PENALTY_TIME	2
 
@@ -37,7 +43,8 @@ class JabberClient::Private
 {
 public:
 	Private()
-	 : jabberClient(0L), jabberClientStream(0L), jabberClientConnector(0L), jabberTLS(0L), jabberTLSHandler(0L)
+	: jabberClient(0L), jabberClientStream(0L), jabberClientConnector(0L), jabberTLS(0L),
+		       jabberTLSHandler(0L), privacyManager(0L)
 	{}
 	~Private()
 	{
@@ -51,6 +58,7 @@ public:
 		delete jabberClientConnector;
 		delete jabberTLSHandler;
 		delete jabberTLS;
+		// privacyManager will be deleted with jabberClient, its parent's parent
 	}
 
 	// connection details
@@ -60,9 +68,11 @@ public:
 	// XMPP backend
 	XMPP::Client *jabberClient;
 	XMPP::ClientStream *jabberClientStream;
-	JabberConnector *jabberClientConnector;
+	XMPP::AdvancedConnector *jabberClientConnector;
 	QCA::TLS *jabberTLS;
 	XMPP::QCATLSHandler *jabberTLSHandler;
+	QCA::Initializer qcaInit;
+	PrivacyManager *privacyManager;
 
 	// ignore TLS warnings
 	bool ignoreTLSWarnings;
@@ -120,9 +130,8 @@ QStringList JabberClient::Private::s5bAddressList;
 int JabberClient::Private::s5bServerPort = 8010;
 
 JabberClient::JabberClient ()
+ : d(new Private())
 {
-	d = new Private();
-
 	cleanUp ();
 
 	// initiate penalty timer
@@ -147,17 +156,19 @@ void JabberClient::cleanUp ()
 	delete d->jabberClientConnector;
 	delete d->jabberTLSHandler;
 	delete d->jabberTLS;
+	// privacyManager will be deleted with jabberClient, its parent's parent
 
 	d->jabberClient = 0L;
 	d->jabberClientStream = 0L;
 	d->jabberClientConnector = 0L;
 	d->jabberTLSHandler = 0L;
 	d->jabberTLS = 0L;
+	d->privacyManager = 0L;
 
 	d->currentPenaltyTime = 0;
 
 	d->jid = XMPP::Jid ();
-	d->password = QString::null;
+	d->password.clear();
 
 	setForceTLS ( false );
 	setUseSSL ( false );
@@ -171,9 +182,9 @@ void JabberClient::cleanUp ()
 	setFileTransfersEnabled ( false );
 	setS5BServerPort ( 8010 );
 
-	setClientName ( QString::null );
-	setClientVersion ( QString::null );
-	setOSName ( QString::null );
+	setClientName ( QString() );
+	setClientVersion ( QString() );
+	setOSName ( QString() );
 
 	setTimeZone ( "UTC", 0 );
 
@@ -270,10 +281,10 @@ void JabberClient::addS5BServerAddress ( const QString &address )
 	d->s5bAddressList.append ( address );
 
 	// now filter the list without dupes
-	for ( QStringList::Iterator it = d->s5bAddressList.begin (); it != d->s5bAddressList.end (); ++it )
+	foreach( QStringList::const_reference str, d->s5bAddressList )
 	{
-		if ( !newList.contains ( *it ) )
-			newList.append ( *it );
+		if ( !newList.contains ( str ) )
+			newList.append ( str );
 	}
 
 	s5bServer()->setHostList ( newList );
@@ -284,11 +295,10 @@ void JabberClient::removeS5BServerAddress ( const QString &address )
 {
 	QStringList newList;
 
-	QStringList::iterator it = d->s5bAddressList.find ( address );
-	if ( it != d->s5bAddressList.end () )
-	{
-		d->s5bAddressList.remove ( it );
-	}
+	int idx = d->s5bAddressList.indexOf( address );
+
+	if ( idx != -1 )
+		d->s5bAddressList.removeAt(idx);
 
 	if ( d->s5bAddressList.isEmpty () )
 	{
@@ -298,10 +308,10 @@ void JabberClient::removeS5BServerAddress ( const QString &address )
 	else
 	{
 		// now filter the list without dupes
-		for ( QStringList::Iterator it = d->s5bAddressList.begin (); it != d->s5bAddressList.end (); ++it )
+		foreach( QStringList::const_reference str, d->s5bAddressList )
 		{
-			if ( !newList.contains ( *it ) )
-				newList.append ( *it );
+			if ( !newList.contains ( str ) )
+				newList.append ( str );
 		}
 
 		s5bServer()->setHostList ( newList );
@@ -545,7 +555,7 @@ XMPP::ClientStream *JabberClient::clientStream () const
 
 }
 
-JabberConnector *JabberClient::clientConnector () const
+XMPP::AdvancedConnector *JabberClient::clientConnector () const
 {
 
 	return d->jabberClientConnector;
@@ -580,6 +590,11 @@ XMPP::FileTransferManager *JabberClient::fileTransferManager () const
 
 }
 
+PrivacyManager *JabberClient::privacyManager () const
+{
+	return d->privacyManager;
+}
+
 XMPP::Jid JabberClient::jid () const
 {
 
@@ -596,15 +611,15 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	{
 		d->jabberClient->close ();
 	}
-
 	d->jid = jid;
 	d->password = password;
 
 	/*
 	 * Return an error if we should force TLS but it's not available.
 	 */
-	if ( ( forceTLS () || useSSL () || probeSSL () ) && !QCA::isSupported ( QCA::CAP_TLS ) )
+	if ( ( forceTLS () || useSSL () || probeSSL () ) && !QCA::isSupported ("tls" ) )
 	{
+		qDebug ("no TLS");
 		return NoTLS;
 	}
 
@@ -613,7 +628,7 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	 * This class uses KDE's socket code, which in turn makes use of
 	 * the global proxy settings.
 	 */
-	d->jabberClientConnector = new JabberConnector;
+	d->jabberClientConnector = new XMPP::AdvancedConnector;
 
 	d->jabberClientConnector->setOptSSL ( useSSL () );
 
@@ -631,18 +646,14 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	/*
 	 * Setup authentication layer
 	 */
-	if ( QCA::isSupported ( QCA::CAP_TLS ) )
+	if ( QCA::isSupported ("tls") )
 	{
 		d->jabberTLS = new QCA::TLS;
-		d->jabberTLSHandler = new XMPP::QCATLSHandler ( d->jabberTLS );
+		d->jabberTLS->setTrustedCertificates(QCA::systemStore());
+		d->jabberTLSHandler = new QCATLSHandler(d->jabberTLS);
+		d->jabberTLSHandler->setXMPPCertCheck(true);
 
-		{
-			using namespace XMPP;
-			QObject::connect ( d->jabberTLSHandler, SIGNAL ( tlsHandshaken() ), this, SLOT ( slotTLSHandshaken () ) );
-		}
-
-		QPtrList<QCA::Cert> certStore;
-		d->jabberTLS->setCertificateStore ( certStore );
+		QObject::connect ( d->jabberTLSHandler, SIGNAL ( tlsHandshaken() ), SLOT ( slotTLSHandshaken () ) );
 	}
 
 	/*
@@ -665,6 +676,8 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 				   this, SLOT ( slotCSWarning (int) ) );
 		QObject::connect ( d->jabberClientStream, SIGNAL ( error (int) ),
 				   this, SLOT ( slotCSError (int) ) );
+		QObject::connect ( d->jabberClientStream, SIGNAL ( connected() ),
+		                   this, SLOT ( slotCSConnected() ) );
 	}
 
 	d->jabberClientStream->setOldOnly ( useXMPP09 () );
@@ -677,12 +690,17 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 	/*
 	 * Allow plaintext password authentication or not?
 	 */
-	d->jabberClientStream->setAllowPlain( allowPlainTextPassword () );
+	d->jabberClientStream->setAllowPlain( allowPlainTextPassword () ?  XMPP::ClientStream::AllowPlain : XMPP::ClientStream::NoAllowPlain );
 
 	/*
 	 * Setup client layer.
 	 */
 	d->jabberClient = new XMPP::Client ( this );
+	
+	/*
+	 * Setup privacy manager
+	 */
+	d->privacyManager = new PrivacyManager ( rootTask() );
 
 	/*
 	 * Enable file transfer (IP and server will be set after connection
@@ -699,12 +717,28 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 		}
 	}
 
+	/*if (jingleEnabled())
+	{*/
+
+#ifdef JINGLE_SUPPORT
+		d->jabberClient->setJingleEnabled(true);
+		
+		{
+			using namespace XMPP;
+			QObject::connect ( d->jabberClient->jingleSessionManager(), SIGNAL ( incomingSession() ),
+					   this, SLOT ( slotIncomingJingleSession () ) );
+		}
+#else
+		d->jabberClient->setJingleEnabled(false);
+#endif
+	/*}*/
+
 	/* This should only be done here to connect the signals, otherwise it is a
 	 * bad idea.
 	 */
 	{
 		using namespace XMPP;
-		QObject::connect ( d->jabberClient, SIGNAL ( subscription (const Jid &, const QString &) ),
+		QObject::connect ( d->jabberClient, SIGNAL ( subscription (const Jid &, const QString &, const QString &) ),
 				   this, SLOT ( slotSubscription (const Jid &, const QString &) ) );
 		QObject::connect ( d->jabberClient, SIGNAL ( rosterRequestFinished ( bool, int, const QString & ) ),
 				   this, SLOT ( slotRosterRequestFinished ( bool, int, const QString & ) ) );
@@ -749,6 +783,9 @@ JabberClient::ErrorCode JabberClient::connect ( const XMPP::Jid &jid, const QStr
 
 	d->jabberClient->setTimeZone ( timeZoneName (), timeZoneOffset () );
 
+	// Additional features
+	d->jabberClient->setFeatures(Features("http://jabber.org/protocol/xhtml-im"));
+
 	d->jabberClient->connectToServer ( d->jabberClientStream, jid, auth );
 
 	return Ok;
@@ -779,7 +816,7 @@ void JabberClient::disconnect( XMPP::Status &reason )
             reason.setIsAvailable( false );
             pres->pres( reason );
             pres->go();
-            
+
             d->jabberClientStream->close();
             d->jabberClient->close();
         }
@@ -811,7 +848,6 @@ void JabberClient::joinGroupChat ( const QString &host, const QString &room, con
 
 void JabberClient::joinGroupChat ( const QString &host, const QString &room, const QString &nick, const QString &password )
 {
-
 	client()->groupChatJoin ( host, room, nick, password );
 
 }
@@ -874,7 +910,7 @@ void JabberClient::slotIncomingXML ( const QString & _msg )
 	msg = msg.replace( QRegExp( "<digest>[^<]*</digest>\n" ), "<digest>[Filtered]</digest>\n" );
 
 	emit debugMessage ( "XML IN: " + msg );
-
+	emit incomingXML ( msg );
 }
 
 void JabberClient::slotOutgoingXML ( const QString & _msg )
@@ -885,7 +921,7 @@ void JabberClient::slotOutgoingXML ( const QString & _msg )
 	msg = msg.replace( QRegExp( "<digest>[^<]*</digest>\n" ), "<digest>[Filtered]</digest>\n" );
 
 	emit debugMessage ( "XML OUT: " + msg );
-
+	emit outgoingXML ( msg );
 }
 
 void JabberClient::slotTLSHandshaken ()
@@ -894,11 +930,13 @@ void JabberClient::slotTLSHandshaken ()
 	emit debugMessage ( "TLS handshake done, testing certificate validity..." );
 
 	// FIXME: in the future, this should be handled by KDE, not QCA
-	int validityResult = d->jabberTLS->certificateValidityResult ();
 
-	if ( validityResult == QCA::TLS::Valid )
+	QCA::TLS::IdentityResult identityResult = d->jabberTLS->peerIdentityResult();
+	QCA::Validity            validityResult = d->jabberTLS->peerCertificateValidity();
+
+	if ( identityResult == QCA::TLS::Valid && validityResult == QCA::ValidityGood )
 	{
-		emit debugMessage ( "Certificate is valid, continuing." );
+		emit debugMessage ( "Identity and certificate valid, continuing." );
 
 		// valid certificate, continue
 		d->jabberTLSHandler->continueAfterHandshake ();
@@ -914,7 +952,7 @@ void JabberClient::slotTLSHandshaken ()
 			d->jabberTLSHandler->continueAfterHandshake ();
 		}
 
-		emit tlsWarning ( validityResult );
+		emit tlsWarning ( identityResult, validityResult );
 	}
 
 }
@@ -968,13 +1006,6 @@ void JabberClient::slotCSAuthenticated ()
 		{
 			d->localAddress = ( (BSocket *)irisByteStream )->address().toString ();
 		}
-
-		// code for the KDE-type bytestream
-		JabberByteStream *kdeByteStream = dynamic_cast<JabberByteStream*>(d->jabberClientConnector->stream());
-		if ( kdeByteStream )
-		{
-			d->localAddress = kdeByteStream->socket()->localAddress().nodeName ();
-		}
 	}
 
 	if ( fileTransfersEnabled () )
@@ -984,12 +1015,35 @@ void JabberClient::slotCSAuthenticated ()
 		d->jabberClient->s5bManager()->setServer ( s5bServer () );
 	}
 
+	
+	//update the ressource:
+	d->jid = d->jabberClientStream->jid();
+
 	// start the client operation
 	d->jabberClient->start ( jid().domain (), jid().node (), d->password, jid().resource () );
 
-	emit connected ();
+	if (!d->jabberClientStream->old())
+	{
+		XMPP::JT_Session *j = new XMPP::JT_Session(rootTask());
+		QObject::connect(j, SIGNAL(finished()), this, SLOT(slotSessionStarted()));
+		j->go(true);
+	}
+	else
+	{
+		emit connected();
+	}
 }
 
+void JabberClient::slotSessionStarted()
+{
+	XMPP::JT_Session *j = static_cast<XMPP::JT_Session*>(sender());
+	if ( j->success() )
+		emit connected();
+	else
+		emit csError ( -1 );
+}
+
+
 void JabberClient::slotCSDisconnected ()
 {
 
@@ -1042,6 +1096,18 @@ void JabberClient::slotCSError ( int error )
 
 }
 
+void JabberClient::slotCSConnected ()
+{
+	Kopete::SocketTimeoutWatcher* timeoutWatcher = 0;
+
+	ByteStream *irisByteStream = d->jabberClientConnector->stream ();
+	if ( irisByteStream && irisByteStream->abstractSocket () )
+		timeoutWatcher = Kopete::SocketTimeoutWatcher::watch ( irisByteStream->abstractSocket () );
+
+	if ( timeoutWatcher )
+		QObject::connect( timeoutWatcher, SIGNAL(errorInt(int)), this, SLOT(slotCSError(int)) );
+}
+
 void JabberClient::slotRosterRequestFinished ( bool success, int /*statusCode*/, const QString &/*statusString*/ )
 {
 
