@@ -21,6 +21,7 @@
 
 #include "xmpp_tasks.h"
 #include "im.h"
+//#include "td.h"
 #include "tasks/jt_getlastactivity.h"
 
 #include <qtimer.h>
@@ -31,6 +32,7 @@
 #include <qbuffer.h>
 #include <QList>
 
+#include <KActionCollection>
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -49,6 +51,7 @@
 #include "kopetechatsessionmanager.h"
 #include "kopeteaccountmanager.h"
 #include "kopetemetacontact.h"
+#include "kopetepluginmanager.h"
 #include "jabberprotocol.h"
 #include "jabberaccount.h"
 #include "jabberclient.h"
@@ -59,10 +62,8 @@
 #include "jabbertransport.h"
 #include "dlgjabbervcard.h"
 
-#ifdef SUPPORT_JINGLE
-// #include "jinglesessionmanager.h"
-// #include "jinglevoicesession.h"
-#include "jinglevoicesessiondialog.h"
+#ifdef JINGLE_SUPPORT
+#include "jinglecallsmanager.h"
 #endif
 
 /**
@@ -71,9 +72,11 @@
 JabberContact::JabberContact (const XMPP::RosterItem &rosterItem, Kopete::Account *_account, Kopete::MetaContact * mc, const QString &legacyId)
 	: JabberBaseContact ( rosterItem, _account, mc, legacyId)  , mDiscoDone(false), m_syncTimer(0L)
 {
+	///*account()->client()->clientStream()->*/XMPP::setDebug((Debug*) new TD());
 	kDebug(JABBER_DEBUG_GLOBAL) << contactId() << "  is created  - " << this;
 	// this contact is able to transfer files
 	setFileCapable ( true );
+	
 
 	/*
 	 * Catch when we're going online for the first time to
@@ -114,7 +117,8 @@ JabberContact::JabberContact (const XMPP::RosterItem &rosterItem, Kopete::Accoun
 		 */
 		if ( account()->myself()->onlineStatus().isDefinitelyOnline() )
 		{
-			slotGetTimedVCard ();
+			mVCardUpdateInProgress = true;
+			QTimer::singleShot ( 1000, this, SLOT ( slotGetTimedVCard () ) );
 		}
 	}
 
@@ -132,7 +136,7 @@ JabberContact::~JabberContact()
 QList<KAction*> *JabberContact::customContextMenuActions ()
 {
 
-	QList<KAction*> *actionCollection = new QList<KAction*>();
+	QList<KAction*> *actions = new QList<KAction*>();
 
 	KActionMenu *actionAuthorization = new KActionMenu ( KIcon("network-connect"), i18n ("Authorization"), this);
 
@@ -159,6 +163,22 @@ QList<KAction*> *JabberContact::customContextMenuActions ()
 	connect(removeAuthAction, SIGNAL(triggered(bool)), SLOT(slotRemoveAuth()));
 	actionAuthorization->addAction(removeAuthAction);
 
+#ifdef GOOGLETALK_SUPPORT
+
+	if ( account()->enabledGoogleTalk() ) {
+
+		KAction *googleTalkCallAction;
+		googleTalkCallAction = new KAction( this );
+		googleTalkCallAction->setIcon( (KIcon("voicecall") ) );
+		googleTalkCallAction->setText( i18n ("Call Google Talk contact") );
+		googleTalkCallAction->setEnabled( account()->supportGoogleTalk(contactId()) );
+		connect(googleTalkCallAction, SIGNAL(triggered(bool)), SLOT(makeGoogleTalkCallAction()));
+		actions->append(googleTalkCallAction);
+
+	}
+
+#endif
+
 	KActionMenu *actionSetAvailability = new KActionMenu ( KIcon("user-identity", 0, QStringList() << QString() << "user-online"), i18n ("Set Availability"), this );
 
 #define KACTION(status, text, name, slot) \
@@ -241,26 +261,40 @@ QList<KAction*> *JabberContact::customContextMenuActions ()
 
 	}
 
-	actionCollection->append( actionAuthorization );
-	actionCollection->append( actionSetAvailability );
-	actionCollection->append( actionSelectResource );
+	actions->append( actionAuthorization );
+	actions->append( actionSetAvailability );
+	actions->append( actionSelectResource );
 	
 	
-#ifdef SUPPORT_JINGLE
-	KAction *actionVoiceCall = new KAction( (i18n ("Voice call"), "voicecall", 0, this, SLOT (voiceCall ()), 0, "jabber_voicecall");
-	actionVoiceCall->setEnabled( false );
+#if 0
+	KAction *testAction = new KAction(i18n("Test action"), this);
+	actionJingleAudioCall->setEnabled( true );
+	actionCollection->append( testAction );
 
-	actionCollection->append( actionVoiceCall );
+	KAction *actionJingleAudioCall = new KAction(i18n("Jingle Audio call"), this);
+	connect(actionJingleAudioCall, SIGNAL(triggered(bool)), SLOT(slotJingleAudioCall()));
+	
+	KAction *actionJingleVideoCall = new KAction(i18n("Jingle Video call"), this);
+	connect(actionJingleVideoCall, SIGNAL(triggered(bool)), SLOT(slotJingleVideoCall()));
 
-	// Check if the current contact support Voice calls, also honor lock by default.
+	// Check if the current contact support jingle calls, also honor lock by default.
 	JabberResource *bestResource = account()->resourcePool()->bestJabberResource( mRosterItem.jid() );
-	if( bestResource && bestResource->features().canVoice() )
-	{
-		actionVoiceCall->setEnabled( true );
-	}
-#endif
+	actionJingleAudioCall->setEnabled( bestResource->features().canJingleAudio() );
+	actionJingleVideoCall->setEnabled( bestResource->features().canJingleVideo() );
 	
-	return actionCollection;
+	actionCollection->append( actionJingleAudioCall );
+	actionCollection->append( actionJingleVideoCall );
+#endif
+
+	// temporary action collection, used to apply Kiosk policy to the actions
+	KActionCollection tempCollection((QObject*)0);
+	tempCollection.addAction(QLatin1String("jabberContactAuthorizationMenu"), actionAuthorization);
+	tempCollection.addAction(QLatin1String("contactSendAuth"), resendAuthAction);
+	tempCollection.addAction(QLatin1String("contactRequestAuth"), requestAuthAction);
+	tempCollection.addAction(QLatin1String("contactRemoveAuth"), removeAuthAction);
+	tempCollection.addAction(QLatin1String("jabberContactSetAvailabilityMenu"), actionSetAvailability);
+	tempCollection.addAction(QLatin1String("jabberContactSelectResource"), actionSelectResource);
+	return actions;
 }
 
 void JabberContact::handleIncomingMessage (const XMPP::Message & message)
@@ -282,8 +316,8 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 			QString originalBody=message.body().isEmpty() ? QString() :
 					i18n( "The original message is : <i>\" %1 \"</i><br />" , Qt::escape(message.body()));
 			QString mes=i18n("<qt><i>%1</i> has invited you to join the conference <b>%2</b><br />%3<br />"
-					"If you want to accept and join, just <b>enter your nickname</b> and press ok<br />"
-							 "If you want to decline, press cancel</qt>",
+					"If you want to accept and join, just <b>enter your nickname</b> and press OK.<br />"
+							 "If you want to decline, press Cancel.</qt>",
 					message.from().full(), room , originalBody);
 			
 			bool ok=false;
@@ -293,7 +327,7 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 				return;
 			
 			XMPP::Jid roomjid(room);
-			account()->client()->joinGroupChat( roomjid.host() , roomjid.user() , futureNewNickName );
+			account()->client()->joinGroupChat( roomjid.domain() , roomjid.node() , futureNewNickName );
 			return;
 		}
 		else if (message.body().isEmpty())
@@ -306,10 +340,15 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 			if (message.containsEvent ( XMPP::DisplayedEvent ) )
 				mManager->receivedEventNotification ( i18n("Message has been displayed") );
 			else if (message.containsEvent ( XMPP::DeliveredEvent ) )
+			{
 				mManager->receivedEventNotification ( i18n("Message has been delivered") );
+				mManager->receivedMessageState( message.eventId().toUInt(), Kopete::Message::StateSent );
+				mSendsDeliveredEvent = true;
+			}
 			else if (message.containsEvent ( XMPP::OfflineEvent ) )
 			{
-	        	mManager->receivedEventNotification( i18n("Message stored on the server, contact offline") );
+				mManager->receivedEventNotification( i18n("Message stored on the server, contact offline") );
+				mManager->receivedMessageState( message.eventId().toUInt(), Kopete::Message::StateSent );
 			}
 			else if (message.chatState() == XMPP::StateGone )
 			{
@@ -319,7 +358,10 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 					m.setPlainBody( i18n("%1 has ended his/her participation in the chat session.", metaContact()->displayName()) );
 					m.setDirection( Kopete::Message::Internal );
 
-					mManager->appendMessage ( m, message.from().resource () );
+					if ( account()->mergeMessages() )
+						mManager->appendMessage ( m, QString() );
+					else
+						mManager->appendMessage ( m, message.from().resource () );
 				}
 			}
 		}
@@ -352,6 +394,7 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 	// check for errors
 	if ( message.type () == "error" )
 	{
+		mManager->receivedMessageState( message.id().toUInt(), Kopete::Message::StateError );
 		newMessage = new Kopete::Message( this, contactList );
 		newMessage->setTimestamp( message.timeStamp() );
 		newMessage->setPlainBody( i18n("Your message could not be delivered: \"%1\", Reason: \"%2\"", 
@@ -370,9 +413,15 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 		QString body = message.body ();
 		if( !message.xencrypted().isEmpty() )
 		{
-			body = QString ("-----BEGIN PGP MESSAGE-----\n\n") + message.xencrypted () + QString ("\n-----END PGP MESSAGE-----\n");
+			kDebug ( JABBER_DEBUG_GLOBAL ) << "Received encrypted message";
+			if (Kopete::PluginManager::self()->plugin("kopete_cryptography"))
+			{
+				kDebug( JABBER_DEBUG_GLOBAL ) << "Kopete cryptography plugin loaded";
+				body = QString ("-----BEGIN PGP MESSAGE-----\n\n") + message.xencrypted () + QString ("\n-----END PGP MESSAGE-----\n");
+			}
 		}
-		else if( message.containsHTML() )
+
+		if( message.containsHTML() )
 		{
 			kDebug ( JABBER_DEBUG_GLOBAL ) << "Received a xHTML message";
 			newMessage = new Kopete::Message ( this, contactList );
@@ -397,7 +446,10 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 	// append message to (eventually new) manager and preselect the originating resource
 	if ( newMessage )
 	{
-		mManager->appendMessage ( *newMessage, message.from().resource () );
+		if ( account()->mergeMessages() )
+			mManager->appendMessage ( *newMessage, QString() );
+		else
+			mManager->appendMessage ( *newMessage, message.from().resource () );
 
 		delete newMessage;
 	}
@@ -421,7 +473,10 @@ void JabberContact::handleIncomingMessage (const XMPP::Message & message)
 		msg.setDirection( Kopete::Message::Inbound );
 		msg.setRequestedPlugin( viewPlugin );
 
-		mManager->appendMessage ( msg, message.from().resource () );
+		if ( account()->mergeMessages() )
+			mManager->appendMessage ( msg, QString() );
+		else
+			mManager->appendMessage ( msg, message.from().resource () );
 	}
 }
 
@@ -837,7 +892,7 @@ JabberChatSession *JabberContact::manager ( Kopete::ContactPtrList chatMembers,
 		 * will stay empty.
 		 */
 		if ( jid.resource().isEmpty () )
-			jid.setResource ( account()->resourcePool()->lockedResource ( jid ).name () );
+			jid = jid.withResource ( account()->resourcePool()->lockedResource ( jid ).name () );
 
 		kDebug(JABBER_DEBUG_GLOBAL) << "No manager found, creating a new one with resource '" << jid.resource () << "'";
 
@@ -1236,46 +1291,6 @@ QString JabberContact::lastReceivedMessageId () const
 	return mLastReceivedMessageId;
 }
 
-void JabberContact::voiceCall( )
-{
-#ifdef SUPPORT_JINGLE
-	Jid jid = mRosterItem.jid();
-	
-	// It's honor lock by default.
-	JabberResource *bestResource = account()->resourcePool()->bestJabberResource( jid );
-	if( bestResource )
-	{
-		if( jid.resource().isEmpty() )
-		{
-			// If the jid resource is empty, get the JID from best resource for this contact.
-			jid = bestResource->jid();
-		}
-	
-		// Check if the voice caller exist and the current resource support voice.
-		if( account()->voiceCaller() && bestResource->features().canVoice() )
-		{
-			JingleVoiceSessionDialog *voiceDialog = new JingleVoiceSessionDialog( jid, account()->voiceCaller() );
-			voiceDialog->show();
-			voiceDialog->start();
-		}
-#if 0
-		if( account()->sessionManager() && bestResource->features().canVoice() )
-		{
-			JingleVoiceSession *session = static_cast<JingleVoiceSession*>(account()->sessionManager()->createSession("http://www.google.com/session/phone", jid));
-
-			JingleVoiceSessionDialog *voiceDialog = new JingleVoiceSessionDialog(session);
-			voiceDialog->show();
-			voiceDialog->start();
-		}
-#endif
-	}
-	else
-	{
-		// Shouldn't never go there.
-	}
-#endif
-}
-
 void JabberContact::slotDiscoFinished( )
 {
 	mDiscoDone = true;
@@ -1296,7 +1311,7 @@ void JabberContact::slotDiscoFinished( )
 				is_transport=true;
 				tr_type=ident.type;
 				//name=ident.name;
-				
+		
 				break;  //(we currently only support gateway)
 			}
 			else if (ident.category == "service")
@@ -1338,12 +1353,72 @@ void JabberContact::slotDiscoFinished( )
 		if(!Kopete::AccountManager::self()->registerAccount( transport ))
 			return;
 		
-		transport->setIdentity( parentAccount->identity() );
 		transport->myself()->setOnlineStatus( status ); //push back the online status
 		return;
 	}
 }
 
+#ifdef JINGLE_SUPPORT
+void JabberContact::showSessionsGui()
+{
+	account()->jingleCallsManager()->showCallsGui();
+}
 
+void JabberContact::startJingleSession()
+{
+	startJingleVideoCall(); //Only to show the message.
+
+	account()->jingleCallsManager()->startNewSession(/*to*/fullAddress());
+	account()->jingleCallsManager()->showCallsGui();
+}
+
+void JabberContact::startJingleAudioCall()
+{
+	startJingleVideoCall(); //Only to show the message.
+
+	//There should also have a list of jingle features supported by the responder client.
+	//--> Will be done in Iris as iris should now what features are supported by the responder client.
+	account()->jingleCallsManager()->startNewSession(/*to*/fullAddress());
+}
+
+void JabberContact::startJingleVideoCall()
+{
+	kDebug() << "Start a Jingle Session";
+
+	//JingleCallsManager should manage messages itself.
+	/*XMPP::Jid jid = rosterItem().jid();
+	JabberResource *bestResource;
+	XMPP::Resource resource = account()->resourcePool()->bestJabberResource( jid )->resource();
+	if (&resource)
+	{
+		JabberChatSession *mManager = manager ( resource.name(), Kopete::Contact::CanCreate );
+		Kopete::Message m = Kopete::Message ( this, mManager->members());
+		m.setPlainBody( i18n("Starting a Jingle session with %1", metaContact()->displayName()) );
+		m.setDirection( Kopete::Message::Internal );
+		mManager->appendMessage ( m, resource.name() );
+	}
+	else
+	{
+		if (!manager(CannotCreate))
+			return;
+		Kopete::Message msg;
+		msg.setPlainBody( "Failed to start a Jingle session, is your contact Online ?" );
+		manager(CannotCreate)->appendMessage( msg );
+		;
+		//FIXME:How to write a message in the chat dialog when contact is offline ?
+	}*/
+
+	//TODO:implement me !
+}
+#endif
+
+#ifdef GOOGLETALK_SUPPORT
+
+void JabberContact::makeGoogleTalkCallAction()
+{
+	account()->makeGoogleTalkCall(contactId());
+}
+
+#endif
 
 #include "jabbercontact.moc"
