@@ -2,7 +2,7 @@
   oscarcontact.cpp  -  Oscar Protocol Plugin
 
   Copyright (c) 2002 by Tom Linsky <twl6@po.cwru.edu>
-  Kopete    (c) 2002-2003 by the Kopete developers  <kopete-devel@kde.org>
+  Kopete    (c) 2002-2008 by the Kopete developers  <kopete-devel@kde.org>
 
   *************************************************************************
   *                                                                       *
@@ -21,6 +21,11 @@
 #include <qapplication.h>
 #include <qtextcodec.h>
 #include <qtimer.h>
+#include <QtXml/QDomDocument>
+#include <QtXml/QDomNodeList>
+#include <QtGui/QTextDocument>
+#include <QtGui/QTextCharFormat>
+#include <QtGui/QTextBlock>
 
 #include <kaction.h>
 #include <kdebug.h>
@@ -51,6 +56,7 @@
 #include "oscarprotocol.h"
 #include "oscarencodingselectiondialog.h"
 #include "oscarstatusmanager.h"
+#include "filetransferhandler.h"
 
 #include <assert.h>
 
@@ -72,6 +78,10 @@ OscarContact::OscarContact( Kopete::Account* account, const QString& name,
 	                  this, SLOT(requestBuddyIcon()) );
 	QObject::connect( mAccount->engine(), SIGNAL(receivedAwayMessage(const QString&, const QString& )),
 	                  this, SLOT(receivedStatusMessage(const QString&, const QString&)) );
+	QObject::connect( mAccount->engine(), SIGNAL(messageAck(const QString&, uint)),
+	                  this, SLOT(messageAck(const QString&, uint)) );
+	QObject::connect( mAccount->engine(), SIGNAL(messageError(const QString&, uint)),
+	                  this, SLOT(messageError(const QString&, uint)) );
 }
 
 OscarContact::~OscarContact()
@@ -187,7 +197,8 @@ void OscarContact::userInfoUpdated( const QString& contact, const UserDetails& d
 	if ( details.buddyIconHash().size() > 0 && details.buddyIconHash() != m_details.buddyIconHash() )
 	{
 		OscarProtocol *p = static_cast<OscarProtocol*>(protocol());
-		if ( property( p->buddyIconHash ).value().toByteArray() != details.buddyIconHash() )
+		QString photoPath = property( Kopete::Global::Properties::self()->photo() ).value().toString();
+		if ( property( p->buddyIconHash ).value().toByteArray() != details.buddyIconHash() || QFileInfo(photoPath).size() == 0 )
 		{
 			m_buddyIconDirty = true;
 			
@@ -271,6 +282,26 @@ void OscarContact::slotTyping( bool typing )
 		account()->engine()->sendTyping( contactId(), typing );
 }
 
+void OscarContact::messageAck( const QString& contact, uint messageId )
+{
+	if ( Oscar::normalize( contact ) != Oscar::normalize( contactId() ) )
+		return;
+	
+	Kopete::ChatSession* chatSession = manager();
+	if ( chatSession )
+		chatSession->receivedMessageState( messageId, Kopete::Message::StateSent );
+}
+
+void OscarContact::messageError( const QString& contact, uint messageId )
+{
+	if ( Oscar::normalize( contact ) != Oscar::normalize( contactId() ) )
+		return;
+	
+	Kopete::ChatSession* chatSession = manager();
+	if ( chatSession )
+		chatSession->receivedMessageState( messageId, Kopete::Message::StateError );
+}
+
 QTextCodec* OscarContact::contactCodec() const
 {
 	if ( hasProperty( "contactEncoding" ) )
@@ -333,8 +364,21 @@ void OscarContact::sendFile( const KUrl &sourceURL, const QString &altFileName,
 	}
 	kDebug(OSCAR_GEN_DEBUG) << "files: '" << files << "' ";
 
-	Kopete::Transfer *t = Kopete::TransferManager::transferManager()->addTransfer( this, files.at(0), QFile( files.at(0) ).size(), mName, Kopete::FileTransferInfo::Outgoing);
-	mAccount->engine()->sendFiles( mName, files, t );
+	FileTransferHandler *ftHandler = mAccount->engine()->createFileTransfer( mName, files );
+
+	Kopete::TransferManager *transferManager = Kopete::TransferManager::transferManager();
+	Kopete::Transfer *transfer = transferManager->addTransfer( this, files, ftHandler->totalSize(), mName, Kopete::FileTransferInfo::Outgoing);
+
+	connect( transfer, SIGNAL(transferCanceled()), ftHandler, SLOT(cancel()) );
+
+	connect( ftHandler, SIGNAL(transferCancelled()), transfer, SLOT(slotCancelled()) );
+	connect( ftHandler, SIGNAL(transferError(int, const QString&)), transfer, SLOT(slotError(int, const QString&)) );
+	connect( ftHandler, SIGNAL(transferProcessed(unsigned int)), transfer, SLOT(slotProcessed(unsigned int)) );
+	connect( ftHandler, SIGNAL(transferFinished()), transfer, SLOT(slotComplete()) );
+	connect( ftHandler, SIGNAL(transferNextFile(const QString&, const QString&)),
+	         transfer, SLOT(slotNextFile(const QString&, const QString&)) );
+
+	ftHandler->send();
 }
 
 void OscarContact::setAwayMessage( const QString &message )
@@ -373,6 +417,125 @@ void OscarContact::requestAuthorization()
 		mAccount->engine()->requestAuth( contactId(), reason );
 }
 
+void OscarContact::slotSendMsg(Kopete::Message& message, Kopete::ChatSession *)
+{
+	if (message.plainBody().isEmpty()) // no text, do nothing
+		return;
+	//okay, now we need to change the message.escapedBody from real HTML to aimhtml.
+	//looking right now for docs on that "format".
+	//looks like everything except for alignment codes comes in the format of spans
+
+	//font-style:italic -> <i>
+	//font-weight:600 -> <b> (anything > 400 should be <b>, 400 is not bold)
+	//text-decoration:underline -> <u>
+	//font-family: -> <font face="">
+	//font-size:xxpt -> <font ptsize=xx>
+
+	QTextDocument doc;
+	doc.setHtml( message.escapedBody() );
+
+	QString rtfText = QString( "<HTML><BODY dir=\"%1\">" ).arg( message.isRightToLeft() ? "rtl" : "ltr" );
+
+	bool hasFontTag = false;
+	QTextCharFormat defaultCharFormat;
+	for ( QTextBlock it = doc.begin(); it != doc.end(); it = it.next() )
+	{
+		QTextBlockFormat blockFormat = it.blockFormat();
+
+		// Plain text message has p tags without margin attributes and Qt's topMargin()
+		// returns default margins so we will end up with line break before text.
+		if ( message.format() != Qt::PlainText || it.blockNumber() != 0 )
+			rtfText += brMargin( blockFormat.topMargin(), defaultCharFormat.fontPointSize() );
+
+		bool lastFragmentHasLineSeparator = false;
+		for ( QTextBlock::iterator it2 = it.begin(); !(it2.atEnd()); ++it2 )
+		{
+			QTextFragment currentFragment = it2.fragment();
+			if ( currentFragment.isValid() )
+			{
+				QTextCharFormat format = currentFragment.charFormat();
+				if ( format.fontFamily() != defaultCharFormat.fontFamily() ||
+				     format.foreground() != defaultCharFormat.foreground() ||
+				     oscarFontSize(format.fontPointSize()) != oscarFontSize(defaultCharFormat.fontPointSize()) )
+				{
+					if ( hasFontTag )
+					{
+						rtfText += "</FONT>";
+						hasFontTag = false;
+					}
+
+					QString fontTag;
+					if ( !format.fontFamily().isEmpty() )
+						fontTag += QString( " FACE=\"%1\"" ).arg( format.fontFamily() );
+					if ( format.fontPointSize() > 0 )
+						fontTag += QString( " SIZE=%1" ).arg( oscarFontSize( format.fontPointSize() ) );
+					if ( format.foreground().style() != Qt::NoBrush )
+						fontTag += QString( " COLOR=%1" ).arg( format.foreground().color().name() );
+					if ( format.background().style() != Qt::NoBrush )
+						fontTag += QString( " BACK=%1" ).arg( format.background().color().name() );
+
+					if ( !fontTag.isEmpty() )
+					{
+						rtfText += QString("<FONT%1>").arg( fontTag );
+						hasFontTag = true;
+					}
+				}
+
+				if ( format.font().bold() != defaultCharFormat.font().bold() )
+					rtfText += ( format.font().bold() ) ? "<B>" : "</B>";
+				if ( format.fontItalic() != defaultCharFormat.fontItalic() )
+					rtfText += ( format.hasProperty(QTextFormat::FontItalic) ) ? "<I>" : "</I>";
+				if ( format.fontUnderline() != defaultCharFormat.fontUnderline() )
+					rtfText += ( format.hasProperty(QTextFormat::FontUnderline) ) ? "<U>" : "</U>";
+
+				QString text = currentFragment.text();
+				lastFragmentHasLineSeparator = text.endsWith( QChar::LineSeparator );
+				rtfText += Qt::escape( text );
+				defaultCharFormat = format;
+			}
+		}
+		rtfText += brMargin( blockFormat.bottomMargin(), defaultCharFormat.fontPointSize(), !lastFragmentHasLineSeparator );
+	}
+
+	rtfText.replace( QChar::LineSeparator, "<BR>" );
+
+	if ( rtfText.endsWith( "<BR>" ) )
+		rtfText.chop(4);
+
+	if ( hasFontTag )
+		rtfText += "</FONT>";
+	if ( defaultCharFormat.font().bold() )
+		rtfText += "</B>";
+	if ( defaultCharFormat.hasProperty( QTextFormat::FontItalic ) )
+		rtfText += "</I>";
+	if ( defaultCharFormat.hasProperty( QTextFormat::FontUnderline ) )
+		rtfText += "</U>";
+
+	rtfText += "</BODY></HTML>";
+
+	kDebug(OSCAR_GEN_DEBUG) << "sending: " << rtfText;
+
+	// TODO: Need to check for message size?
+
+	Oscar::Message msg;
+	// Allow UCS2 because official AIM client doesn't sets the CAP_UTF8 anymore!
+	bool allowUCS2 = !isOnline() || !(m_details.userClass() & Oscar::CLASS_ICQ) || m_details.hasCap( CAP_UTF8 );
+	msg.setText( Oscar::Message::encodingForText( rtfText, allowUCS2 ), rtfText, contactCodec() );
+
+	msg.setId( message.id() );
+	msg.setReceiver(mName);
+	msg.setSender( mAccount->accountId() );
+	msg.setTimestamp(message.timestamp());
+	msg.setChannel(0x01);
+
+	mAccount->engine()->sendMessage(msg);
+
+	message.setState( Kopete::Message::StateSending );
+	// Show the message we just sent in the chat window
+	manager(Kopete::Contact::CanCreate)->appendMessage(message);
+	manager(Kopete::Contact::CanCreate)->messageSucceeded();
+}
+
 void OscarContact::changeEncodingDialogClosed( int result )
 {
 	if ( result == QDialog::Accepted )
@@ -413,11 +576,11 @@ void OscarContact::haveIcon( const QString& user, QByteArray icon )
 		entry.image = img;
 		entry = Kopete::AvatarManager::self()->add(entry);
 
-		setProperty( static_cast<OscarProtocol*>(protocol())->buddyIconHash, m_details.buddyIconHash() );
-		if (!entry.path.isNull())
+		if (!entry.dataPath.isNull())
 		{
 			removeProperty( Kopete::Global::Properties::self()->photo() );
-			setProperty( Kopete::Global::Properties::self()->photo(), entry.path );
+			setProperty( Kopete::Global::Properties::self()->photo(), entry.dataPath );
+			setProperty( static_cast<OscarProtocol*>(protocol())->buddyIconHash, m_details.buddyIconHash() );
 		}
 
 		m_buddyIconDirty = false;
@@ -425,6 +588,7 @@ void OscarContact::haveIcon( const QString& user, QByteArray icon )
 	else
 	{
 		kDebug(14153) << "Buddy icon hash does not match!";
+		removeProperty( static_cast<OscarProtocol*>(protocol())->buddyIconHash );
 		removeProperty( Kopete::Global::Properties::self()->photo() );
 	}
 }
@@ -453,5 +617,40 @@ QString OscarContact::filterAwayMessage( const QString &message ) const
 	return filteredMessage;
 }
 
+int OscarContact::oscarFontSize( int size ) const
+{
+	if ( size <= 0 )
+		return 0;
+	else if ( 1 <= size && size <= 9 )
+		return 1;
+	else if ( 10 <= size && size <= 11 )
+		return 2;
+	else if ( 12 <= size && size <= 13 )
+		return 3;
+	else if ( 14 <= size && size <= 16 )
+		return 4;
+	else if ( 17 <= size && size <= 22 )
+		return 5;
+	else if ( 23 <= size && size <= 29 )
+		return 6;
+	else
+		return 7;
+}
+
+QString OscarContact::brMargin( int margin, int fontPointSize, bool forceBr ) const
+{
+	int brHeight = ( fontPointSize == 0 ) ? 12 : fontPointSize;
+	int brCount = margin / brHeight;
+	
+	if ( brCount <= 0 )
+		return ( forceBr ) ? "<BR>" : "";
+	
+	QString s;
+	while ( brCount-- > 0 )
+		s += "<BR>";
+	
+	return s;
+}
+
 #include "oscarcontact.moc"
 //kate: tab-width 4; indent-mode csands;
