@@ -1,25 +1,27 @@
-/*  *************************************************************************
-    *   copyright: (C) 2003 Richard Lrkng <nouseforaname@home.se>         *
-    *   copyright: (C) 2003 Gav Wood <gav@kde.org>                          *
-    *************************************************************************
+/*
+  smssendprovider.cpp  -  SMS Plugin
+
+  Copyright (c) 2003      by Richard Lärkäng        <nouseforaname@home.se>
+  Copyright (c) 2003      by Gav Wood               <gav@kde.org>
+
+  *************************************************************************
+  *                                                                       *
+  * This program is free software; you can redistribute it and/or modify  *
+  * it under the terms of the GNU General Public License as published by  *
+  * the Free Software Foundation; either version 2 of the License, or     *
+  * (at your option) any later version.                                   *
+  *                                                                       *
+  *************************************************************************
 */
 
-/*  *************************************************************************
-    *                                                                       *
-    * This program is free software; you can redistribute it and/or modify  *
-    * it under the terms of the GNU General Public License as published by  *
-    * the Free Software Foundation; either version 2 of the License, or     *
-    * (at your option) any later version.                                   *
-    *                                                                       *
-    *************************************************************************
-*/
+#include "smssendprovider.h"
 
-#include <qvaluelist.h>
-#include <qlabel.h>
-#include <qfile.h>
+#include <QFile>
+#include <QTextStream>
+#include <QList>
 
 #include <kconfigbase.h>
-#include <kprocess.h>
+#include <k3process.h>
 #include <klineedit.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
@@ -28,14 +30,13 @@
 #include "kopeteaccount.h"
 #include "kopeteuiglobal.h"
 
-#include "smssendprovider.h"
 #include "smsprotocol.h"
 #include "smscontact.h"
 
-SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& prefixValue, Kopete::Account* account, QObject* parent, const char *name)
-	: QObject( parent, name ), m_account(account)
+SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& prefixValue, Kopete::Account* account, QObject* parent)
+	: QObject( parent ), m_account(account)
 {
-	kdWarning( 14160 ) << k_funcinfo << "this = " << this << ", m_account = " << m_account << " (should be ok if zero!!)" << endl;
+	kWarning( 14160 ) << "this = " << this << ", m_account = " << m_account << " (should be ok if zero!!)";
 
 	provider = providerName;
 	prefix = prefixValue;
@@ -46,7 +47,7 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 
 	QString file = prefix + "/share/smssend/" + provider + ".sms";
 	QFile f(file);
-	if (f.open(IO_ReadOnly))
+	if (f.open(QIODevice::ReadOnly))
 	{
 		QTextStream t(&f);
 		QString group = QString("SMSSend-%1").arg(provider);
@@ -63,18 +64,18 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 		numberWords.append("telefone");
 		numberWords.append("ToPhone");
 
-		while( !t.eof())
+		while( !t.atEnd())
 		{
 			QString s = t.readLine();
 			if( s[0] == '%')
 			{
-				QStringList args = QStringList::split(':',s);
-				QStringList options = QStringList::split(' ', args[0]);
+				QStringList args = s.split(':');
+				QStringList options = args[0].split(' ');
 
 				names.append(options[0].replace(0,1,""));
 
 				bool hidden = false;
-				for(unsigned i = 1; i < options.count(); i++)
+				for(int i = 1; i < options.count(); i++)
 					if(options[i] == "Hidden")
 					{	hidden = true;
 						break;
@@ -83,12 +84,12 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 
 				// Strip trailing whitespace in the end
 				// and '%' in the beginning
-				args[0] = args[0].simplifyWhiteSpace().mid(1);
+				args[0] = args[0].simplified().mid(1);
 
 				descriptions.append(args[1]);
 				if (m_account)
 					values.append(m_account->configGroup()->readEntry(QString("%1:%2").arg(group).arg(names[names.count()-1]),
-					                                                  QString::null));
+					                                                  QString()));
 				else
 					values.append("");
 
@@ -96,7 +97,7 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 					|| args[0].contains("message") || args[0].contains("nachricht")
 					|| args[0].contains("Msg") || args[0].contains("Mensagem") )
 				{
-					for( unsigned i = 0; i < options.count(); i++)
+					for( int i = 0; i < options.count(); i++)
 					{
 						if (options[i].contains("Size="))
 						{
@@ -116,10 +117,10 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 							telPos = names.count() - 1;
 							if (args[0] == *it)
 							{
-//								kdDebug(14160) << "Exact match for " << args[0] << endl;
+//								kDebug(14160) << "Exact match for " << args[0];
 								exactNumberMatch = true;
 							}
-//							kdDebug(14160) << "args[0] (" << args[0] << ") contains " << *it << endl;
+//							kDebug(14160) << "args[0] (" << args[0] << ") contains " << *it;
 						}
 					}
 				}
@@ -139,7 +140,7 @@ SMSSendProvider::SMSSendProvider(const QString& providerName, const QString& pre
 
 SMSSendProvider::~SMSSendProvider()
 {
-	kdWarning( 14160 ) << k_funcinfo << "this = " << this << endl;
+	kWarning( 14160 ) << "this = " << this;
 }
 
 void SMSSendProvider::setAccount(Kopete::Account *account)
@@ -147,10 +148,10 @@ void SMSSendProvider::setAccount(Kopete::Account *account)
 	m_account = account;
 }
 
-const QString& SMSSendProvider::name(int i)
+QString SMSSendProvider::name(int i)
 {
 	if ( telPos == i || messagePos == i)
-		return QString::null;
+		return QString();
 	else
 		return names[i];
 }
@@ -165,33 +166,33 @@ const QString& SMSSendProvider::description(int i)
 	return descriptions[i];
 }
 
-const bool SMSSendProvider::isHidden(int i)
+bool SMSSendProvider::isHidden(int i) const
 {
 	return isHiddens[i];
 }
 
-void SMSSendProvider::save(QPtrList<KLineEdit>& args)
+void SMSSendProvider::save(const QList<KLineEdit*>& args)
 {
-	kdDebug( 14160 ) << k_funcinfo << "m_account = " << m_account << " (should be non-zero!!)" << endl;
+	kDebug( 14160 ) << "m_account = " << m_account << " (should be non-zero!!)";
 	if (!m_account) return;		// prevent crash in worst case
 
 	QString group = QString("SMSSend-%1").arg(provider);
 	int namesI=0;
 
-	for (unsigned i=0; i < args.count(); i++)
+	for (int i=0; i < args.count(); i++)
 	{
 		if (telPos == namesI || messagePos == namesI)
 		{
-//		    kdDebug(14160) << k_funcinfo << "Skipping pos " << namesI << endl;
+//		    kDebug(14160) << "Skipping pos " << namesI;
 		    namesI++;
 		    if (telPos == namesI || messagePos == namesI)
 		    {
-//		        kdDebug(14160) << k_funcinfo << "Skipping pos " << namesI << endl;
+//		        kDebug(14160) << "Skipping pos " << namesI;
 		        namesI++;
 		    }
 		}
 
-//                kdDebug(14160) << k_funcinfo << "saving " << args.at(i) << " to " << names[namesI] << endl;
+//                kDebug(14160) << "saving " << args.at(i) << " to " << names[namesI];
 		if (!args.at(i)->text().isEmpty())
 		{	values[namesI] = args.at(i)->text();
 			m_account->configGroup()->writeEntry(QString("%1:%2").arg(group).arg(names[namesI]), values[namesI]);
@@ -212,7 +213,7 @@ void SMSSendProvider::send(const Kopete::Message& msg)
 		if ( messagePos == -1 )
 		{
 			canSend = false;
-			KMessageBox::error(Kopete::UI::Global::mainWidget(), i18n("Could not determine which argument which should contain the message."),
+			KMessageBox::error(Kopete::UI::Global::mainWidget(), i18n("Could not determine which argument should contain the message."),
 				i18n("Could Not Send Message"));
 			return;
 		}
@@ -220,7 +221,7 @@ void SMSSendProvider::send(const Kopete::Message& msg)
 		{
 			canSend = false;
 
-			KMessageBox::error(Kopete::UI::Global::mainWidget(), i18n("Could not determine which argument which should contain the number."),
+			KMessageBox::error(Kopete::UI::Global::mainWidget(), i18n("Could not determine which argument should contain the number."),
 				i18n("Could Not Send Message"));
 			return;
 		}
@@ -237,38 +238,38 @@ void SMSSendProvider::send(const Kopete::Message& msg)
 	values[messagePos] = message;
 	values[telPos] = nr;
 
-	KProcess* p = new KProcess;
+	K3Process* p = new K3Process;
 
-	kdWarning( 14160 ) << "Executing " << QString("%1/bin/smssend").arg(prefix) << " \"" << provider << "\" " << values.join("\" \"") << "\"" << endl;
+	kWarning( 14160 ) << "Executing " << QString("%1/bin/smssend").arg(prefix) << " \"" << provider << "\" " << values.join("\" \"") << "\"";
 
 	*p << QString("%1/bin/smssend").arg(prefix) << provider << values;
 
 	output = "";
-	connect( p, SIGNAL(processExited(KProcess *)), this, SLOT(slotSendFinished(KProcess *)));
-	connect( p, SIGNAL(receivedStdout(KProcess *, char *, int)), this, SLOT(slotReceivedOutput(KProcess *, char *, int)));
-//	connect( p, SIGNAL(receivedStderr(KProcess *, char *, int)), this, SLOT(slotReceivedOutput(KProcess *, char *, int)));
+	connect( p, SIGNAL(processExited(K3Process *)), this, SLOT(slotSendFinished(K3Process *)));
+	connect( p, SIGNAL(receivedStdout(K3Process *, char *, int)), this, SLOT(slotReceivedOutput(K3Process *, char *, int)));
+//	connect( p, SIGNAL(receivedStderr(K3Process *, char *, int)), this, SLOT(slotReceivedOutput(K3Process *, char *, int)));
 
-	p->start(KProcess::NotifyOnExit, KProcess::AllOutput);
+	p->start(K3Process::NotifyOnExit, K3Process::AllOutput);
 }
 
-void SMSSendProvider::slotSendFinished(KProcess *p)
+void SMSSendProvider::slotSendFinished(K3Process *p)
 {
-	kdWarning( 14160 ) << k_funcinfo << "this = " << this << ", es = " << p->exitStatus() << ", p = " << p << " (should be non-zero!!)" << endl;
+	kWarning( 14160 ) << "this = " << this << ", es = " << p->exitStatus() << ", p = " << p << " (should be non-zero!!)";
 	if (p->exitStatus() == 0)
 		emit messageSent(m_msg);
 	else
-		emit messageNotSent(m_msg, QString().setLatin1(output));
+		emit messageNotSent(m_msg, QString::fromLatin1(output));
 
 	p->deleteLater();
 }
 
-void SMSSendProvider::slotReceivedOutput(KProcess *, char *buffer, int buflen)
+void SMSSendProvider::slotReceivedOutput(K3Process *, char *buffer, int buflen)
 {
 //	QStringList lines = QStringList::split("\n", QString::fromLocal8Bit(buffer, buflen));
 //	for (QStringList::Iterator it = lines.begin(); it != lines.end(); ++it)
 	for(int i = 0; i < buflen; i++)
 		output += buffer[i];
-	kdWarning( 14160 ) << k_funcinfo << " output now = " << output << endl;
+	kWarning( 14160 ) << " output now = " << output;
 }
 
 int SMSSendProvider::maxSize()
